MODULE SYSTEMXNS

!   ============================================================
!   Purpose : This module contains common block array definition 
!             and the physisc of the problem  - EoS
!
!   All parameters to be changed by user should be set here.
!
!   ============================================================
!   ============================================================

  !Parameter for convergence 
  INTEGER,PARAMETER :: NVALUE  =100  !Iteration for the Newton scheme in XNS
  INTEGER,PARAMETER :: MAXLOOP =1000 ! Max Number of loops for convergence in XNSMAIN(1 = TOV solution)
  
  ! Paramater for the grid
  INTEGER,PARAMETER :: NR    = 600 ! Radial mesh points
  INTEGER,PARAMETER :: NTH   = 300 ! Angular mesh points (1 = 1D TOV solution)
  
  REAL,PARAMETER :: RMIN = 0.         ! Center
  REAL,PARAMETER :: RMAX = 30.        ! Outer radius
  REAL,PARAMETER :: REQMAX = 15.     ! Truncation radius for equilibrium solution (to avoid explosion)
 
  !Parameter for a stretched grid
  LOGICAL,PARAMETER :: STRETCH =.FALSE. ! If .TRUE. the radial grid is uniform up to RREG
  INTEGER,PARAMETER :: NRREG =600       ! Points for the regular grid
  REAL,PARAMETER :: RREG = 30.          ! Radius for the regular Grid if Stretch=.TRUE.
  REAL :: STRR                          !Stretching Ratio
                                 
  ! Flag for writing output files. Put false for speed test.
  ! Will always wrtite output for the final step of convergence loop
  LOGICAL :: WRT = .TRUE.      ! Write all files
  LOGICAL :: WRTF = .TRUE.     ! Write only final files
  LOGICAL :: VERBOSE = .TRUE.  ! Print all infos 
  LOGICAL :: CHUP = .TRUE.     ! Write all files (including the metric test)
  LOGICAL :: WGRID= .TRUE.     ! Write the grid file
  LOGICAL :: WCONVA= .TRUE.   ! Write file to check convergence of Aphi and Atim
  LOGICAL :: IDAT              ! Write infos on the convergence for QUOC
  
  !Parameter for  the overall Newton-Raphson scheme (if precompiled)
#ifdef NWTRPS  
  INTEGER,PARAMETER :: QUOC=0          !Quantity of convergence for sequences
                                       !QUOC=0 => central density
                                       !QuOC=1 => gravitational mass
                                       !QUOC=2 => baryonic mass
  REAL,PARAMETER :: QUCONV = 0.  !Value of quantity to convergence
#endif

  !Physics parameter - central density
  REAL,PARAMETER :: RHOINI = 7.4431E-4  !Guess for central density (beware scheme converges to QUCONV)

  !Physics parameter - EoS
  REAL,PARAMETER :: K1 = 110.         ! Politropic coefficient
  REAL,PARAMETER :: GAMMA = 2.        ! Politropic exponent

  !Physics parameter - Rotation
  REAL,PARAMETER :: OMG = 5.e-3            ! Central Rotation rate
  LOGICAL        :: DIFFERENTIAL = .FALSE. ! Differential rotation option
  REAL,PARAMETER :: A2VALUE = 0.           ! Differential rotation coeff

  !Physics - Magnetic Fields
  LOGICAL, PARAMETER :: IMAG = .TRUE.    ! Magnetized cases
  LOGICAL, PARAMETER :: ITOR = .FALSE.    ! Purely toroidal B-field
  LOGICAL, PARAMETER :: IPOL = .TRUE.   ! Purely poloidal B-field
  LOGICAL, PARAMETER :: ITWT = .FALSE.   ! Mixed           B-field

  !Physics - purely TOROIDAL B-field only!
  REAL, PARAMETER :: BCOEF  = 0.      !Toroidal magnetization constant
  REAL, PARAMETER :: MAGIND = 0.      !Toroidal magnetization index (>=1)

  !Physics - purely POLOIDAL B-field only!
  REAL,PARAMETER :: KBPOL = 0.3973242   ! Magnetic coefficient
  !REAL,PARAMETER :: NPOL  = 0.         ! Magnetic powerlaw index  (>= 1)
  REAL,PARAMETER :: CSI = 0.            ! Current coefficient
  LOGICAL, PARAMETER :: QNULL=.TRUE.    ! Logical parameter for the star charge (IF OMG.NE.0)
  LOGICAL, PARAMETER :: CTP=.FALSE.     ! Use cons to prim routines in the case of poloidal B-field

  !Physics - TWISTED-TORUS magnetic field
  REAL, PARAMETER :: KBTT=0.            ! Magnetic coefficient
  REAL, PARAMETER :: ATWT=0.            ! TT magnetic coefficient
  REAL, PARAMETER :: ZETA=1.            ! TT magnetic index
  REAL, PARAMETER :: CUT=1.             ! Maximum distance for twisted magnetosphere

  !REAL :: BCOEFT, KBTT,KBPOL

  ! Parameters to help or stabilize convergence
  REAL, PARAMETER    :: CONVF= 1.d-4     ! Convergence of the newton scheme in XNS
  REAL, PARAMETER    :: QFACTOR=0.5      ! Damping factor for the convergence loop qnew = qfactro*qnew + (1-qfactor)*qold
  REAL, PARAMETER    :: QAPHI=1.         ! Damping factor for the convergence of the vector potential 
  REAL, PARAMETER    :: EPS=1.E-7

  ! Array for the Legendre expansion
  INTEGER,PARAMETER :: MLS = 40 !20     ! Number of Legendre polinomia for expansion in theta (0 = 1D)
  INTEGER,PARAMETER :: NGQ = 80 !80!60  ! Number of point in the Gauss quadrature (1 = 1D)
  INTEGER,PARAMETER :: MLSL= 20 !10     ! Number of Legendre polinomia used to solve Laplace equation
  REAL,PARAMETER    :: TOLCONV = 1.D-10 ! Convergence for the lapse and conformal factor iterative solvers

  INTEGER :: IX,IZ
  REAL :: PI
  REAL, DIMENSION(0:NR+1)   :: R,DR
  REAL, DIMENSION(-1:NTH+2) :: XX,TH,DTH
  
  ! Paramaters of the fluid field and metric
  REAL, DIMENSION(1:NTH,1:NR) :: PSI     ! Conformal factor
  REAL, DIMENSION(1:NTH,1:NR) :: PSL     ! Lapse * conformal factor
  REAL, DIMENSION(1:NTH,1:NR) :: PSS     ! Phi - shift
  REAL, DIMENSION(1:NTH,1:NR) :: PSSR    ! R - shift
  REAL, DIMENSION(1:NTH,1:NR) :: PSST    ! Theta - shift
  
  REAL, DIMENSION(-1:NTH+2,0:NR+1):: RHOSRC,ESRC,PSRC,VPHI,VR,VTH,BPHI,EPHI,SSS
  REAL, DIMENSION(-1:NTH+2,0:NR+1):: USRC,DSRC,S3SRC,S1SRC,S2SRC
  REAL, DIMENSION(-1:NTH+2,0:NR+1):: ECSRC,ELSRC,ES1SRC,ES2SRC,ES3SRC
  REAL, DIMENSION(-1:NTH+2,0:NR+1):: CURVC,CURVR,CURVT,CURVP
  
  ! Initial TOV solution
  REAL, DIMENSION(1:NR) :: MU,NU,RHOTV,PRTV,ETV

  ! Paramaters of the Equilibrium solution via bernoulli integral
  REAL, DIMENSION(1:NTH,1:NR) :: PNEW,RHONEW,ENEW,V3NEW,B3NEW,E3NEW
  
  REAL :: USR,SYSR,SY2SR,DSR,DENSC,PRESSC,VELOCC,BMAG,B2YSR
  
  INTEGER,DIMENSION(1:NTH) :: WSURF,CUTSURF    !Integer vectors for surfaces
  REAL,DIMENSION(-1:NTH+2) :: ELSURF           !Fitted surface 
  REAL :: NSPE
  
  ! Parameters for the magnetic vector potential
  REAL,DIMENSION(1:NTH,1:NR) :: RHOTERM,OMTERM,VMETERM,VLOC
  REAL,DIMENSION(1:NTH,1:NR) :: JPHIMXL,RHOEMXL
  REAL,DIMENSION(1:NTH,1:NR) :: DRMETTERM,DTMETTERM
  REAL,DIMENSION(1:NTH,1:NR) :: DROMGM,DTOMGM
  REAL,DIMENSION(1:NTH,1:NR) :: DRGAML,DTGAML
  REAL,DIMENSION(1:NTH,1:NR) :: BPOLT,BPOLR,EPOLT,EPOLR,ATHETA
  REAL,DIMENSION(1:NTH,1:NR) :: JPHI, JTH, JRR
  
  REAL,DIMENSION(-1:NTH+2,0:NR+1):: METERM,GAMLOC,LOGGAM,OMGMET
  REAL,DIMENSION(-1:NTH+2,0:NR+1):: APHI,ATIM,ATIMARM,ATIMIN,ATIMOUT

  REAL :: APHIMAX,APM,RFCUT
 
  ! Parameters for covterm
  REAL    :: ALPHA,GP,GM,GCOVR,GCOVT,GCOVP
  INTEGER :: ISUR,ELOOP
  REAL    :: REQCHECK
  
  ! Stuff for parallel-MPI runs (for parameters study)
  CHARACTER(len=50) :: subdirpath
  INTEGER :: XNSERR

CONTAINS

SUBROUTINE EOSTOV(P,RHO,EN)

!   ============================================================
!   Purpose : EoS for the TOV solution - poly-rho
!             rho = rest mass density, e = thermal energy
!   ============================================================

  REAL :: P,RHO,EN
  
  ! Given the pressure derive density and thermal energy
  IF(P .GT. 1.E-15)THEN
     RHO=(P/K1)**(1./GAMMA)
  ELSE   
     P=1.E-15
     RHO=(P/K1)**(1./GAMMA)
  END IF
  
  EN=P/(GAMMA-1.)
  
  RETURN

END SUBROUTINE EOSTOV

! ********************************************************
! ********************************************************

SUBROUTINE FUNCD_EOS(X,Y,DY,RHOVAR)
!   ============================================================
!   Purpose : Used by rtsafe to derive central pressure 
!             given the central density
!   ============================================================
  
  REAL :: X,Y,YN,Z,ZN
  INTEGER :: NN
  
  CALL EOSTOV(X,Y,Z)
  Y=Y-RHOVAR
  
  XN=1.0001*X
  CALL EOSTOV(XN,YN,ZN)
  YN=YN-RHOVAR
  
  DY=(YN-Y)/(XN-X)

  RETURN
  
END SUBROUTINE FUNCD_EOS

! ********************************************************

SUBROUTINE FUNCD_STRETCH(X,Y,DY,RHOVAR)
!   ============================================================
!   Purpose : Used by rtsafe to derive stretching factor 
!             for the grid
!   ============================================================
  
  REAL :: X,Y,YN,Z,ZN
  INTEGER :: NN
  
  NN=(NR-NRREG)
  
  Y=X*(1-X**NN)/(1-X)-NRREG*(RMAX-RREG)/(RREG-RMIN)
  DY= (NN*X**(NN+1.) - (1+NN)*X**NN +1.)/(X-1.)**2.
    
  RETURN

END SUBROUTINE FUNCD_STRETCH

! ***********************************************************************
! **************************** NR routines ****************************** 

FUNCTION RTSAFEG(X0,X1,X2,XACC,RHOVAR,FUNCD)

  INTERFACE 
    SUBROUTINE FUNCD(a,b,c,d)
      real :: a,b,c,d
    END SUBROUTINE FUNCD
  END INTERFACE   

  PARAMETER (MAXIT=2000)

  CALL FUNCD(X1,FL,DF,RHOVAR)
  
  IF(FL.EQ.0.) THEN
     RTSAFEG=X1
     RETURN
  ENDIF
  CALL FUNCD(X2,FH,DF,RHOVAR)

  IF(FH.EQ.0.) THEN
     RTSAFEG=X2
     RETURN
  ENDIF
  IF(FL*FH.GT.0.)THEN
     WRITE(6,*)'FL ',FL ,'FH ',FH
     !STOP 'Root must be bracketed'
     XACC=-1.
     RETURN
  ENDIF
  IF(FL.LT.0.)THEN
     XL=X1
     XH=X2
  ELSE
     XH=X1
     XL=X2
     SWAP=FL
     FL=FH
     FH=SWAP
  ENDIF
  RTSAFEG=X0
  DXOLD=ABS(X2-X1)
  DX=DXOLD
  CALL FUNCD(RTSAFEG,F,DF,RHOVAR)
  DO 11 J=1,MAXIT
     IF(((RTSAFEG-XH)*DF-F)*((RTSAFEG-XL)*DF-F).GE.0. &
          .OR. ABS(2.*F).GT.ABS(DXOLD*DF) ) THEN
        DXOLD=DX
        DX=0.5*(XH-XL)
        RTSAFEG=XL+DX
        IF(XL.EQ.RTSAFEG)RETURN
     ELSE
        DXOLD=DX
        DX=F/DF
        TEMP=RTSAFEG
        RTSAFEG=RTSAFEG-DX
        IF(TEMP.EQ.RTSAFEG)RETURN
     ENDIF
     IF(ABS(DX).LT.XACC) RETURN
     CALL FUNCD(RTSAFEG,F,DF,RHOVAR)
     IF(F.LT.0.) THEN
        XL=RTSAFEG
        FL=F
     ELSE
        XH=RTSAFEG
        FH=F
     ENDIF
11    CONTINUE
     WRITE(6,*) 'RTSAFE exceeding maximum iterations'
     
     XACC=+1
     RETURN
END FUNCTION RTSAFEG

! **************************************************************************

SUBROUTINE XNS2ECHO_OUT()
  
  open(10,file='XNS2ECHO_init.dat',form='unformatted')
  print*, NTH,NR
  write(10), NTH
  write(10), NR
  write(10), RMIN
  write(10), RMAX
  write(10), RHONEW
  write(10), V3NEW
  write(10), PNEW
  write(10), EPOLR
  write(10), EPOLT
  write(10), BPOLR
  write(10), BPOLT
  write(10), B3NEW
  write(10), PSI
  write(10), PSL/PSI
  write(10), PSS
  close(10)

END SUBROUTINE XNS2ECHO_OUT


END MODULE SYSTEMXNS

