; ##########################################################################
; Read the data file prodiuced by XNS
; It visualizes properties of the equilibrium solution.
; ##########################################################################  

nxi=250          ; Number of resampling points for cartesian graphics
colortable=13    ; Standard color table
smoothing=0.1    ; Smoothing value for curves
pcirc=1.e-12     ; Pressure threshold to plot the surface
thickness=1.5    ; Thickness of curves
radconv=0.6672   ; conversion factor for radii
fsz=15


;position of plots inside the window
pos1=[.15,.20,.85,.90]
pos2=[.10,.05,.90,.10]

;definition of arrays 
rx=dblarr(2*nxi)                ;array for the cartesian resampling-x
rx2d=dblarr(2*nxi,2*nxi)        ;2d array for the cartesian resampling-x
ry=dblarr(2*nxi)                ;array for the cartesian resampling-y
ry2d=dblarr(2*nxi,2*nxi)        ;2d array for the cartesian resampling-y
rxA=dblarr(nxi)
rxB=dblarr(nxi)
arr=dblarr(2*nxi,2*nxi)
arraphi=dblarr(2*nxi,2*nxi)
arratim=dblarr(2*nxi,2*nxi)
arrA=dblarr(nxi,2*nxi)
arrB=dblarr(nxi,2*nxi)
arrcirc=dblarr(2*nxi,2*nxi)
arruni=dblarr(2*nxi,2*nxi)

Exx=dblarr(2*nxi,2*nxi)
Eyy=dblarr(2*nxi,2*nxi)
Exx2=dblarr(2*nxi,2*nxi)
Eyy2=dblarr(2*nxi,2*nxi)

device,decompose=0
lun=10
st=''
print,' Please select which option ?'
print,'  1: Density (Linear)'
print,'  2: Density (Log10)'
print,'  3: V^phi'
print,'  4: V^2'
print,'  5: Lorentz Factor'
print,'  6: B_tor(Linear)'
print,'  7: B_tor(Log10)'
print,'  8: B^2/(e+p)'
print,'  9: Alpha'
print,' 10: Psi'
print,' 11: Beta'
print,' 12: Vector potential (aphi)'
print,' 13: Vector potential (rho)'
print,' 14: B_pol (Linear)'
print,' 15: B_tor VS B_pol'
print,' 16: Electric field '
print,' 17: E_pol VS B_pol '
read,' Equation = ? ',field

if(field ne 0) then begin
print,' Please select the Radius for cartesian resampling ?'
read,' Radius = ?',rmax         ; Cartesian resampling radius
rx=(0.5+findgen(2*nxi))/nxi*rmax-rmax
ry=rx

for i=0,2*nxi-1 do begin
  for j=0, 2*nxi-1 do begin
  rx2d(i,j)=rx(i)
  ry2d(i,j)=ry(j)
  endfor
endfor


print,' Please select the output' 
print,'  0: screen'
print,'  1: file  '
read,' output = ? ',output

ntest=0
maxntest=8
endif

if(field eq 0) then begin
      
   print,'-----------------------------------'
   print,'          ~ TEST MODE ~            '
   print,'-----------------------------------'
   print,'Please select option '
   print,'1: Read all avalaible test files'
   print,'2: Check convergence'
   print,'3: Test Euler/Bernoulli equation'
   print,'4: Test Vectorial Maxwell equations'
   print,'5: Test Potential Maxwell equations'
   print,'6: Test MHD'
   print,'7: Check Surface' 
   print,'8: Check primitive '
   print,'>: All tests'
   read, 'Option number = ? ', ntest
      
   maxntest=8
   if(ntest le 0 ) then ntest=1
   output=0
   radconv=1. ;test are made in code units
endif 

; *****************   Hydro Equilib solution   *********************

openr,lun,'Grid.dat'
readf,lun,nx,nz,nn,ss,gamma
print,' Dimensions:   ',nx,nz
print,' Regular grid: ',nn
print,' Stretch:      ',ss
print,' Adiab. Index: ',gamma
gam=gamma
nz=fix(nz)
nx=fix(nx)
nn=fix(nn)
r=dblarr(nz)
th=dblarr(nx)
rflag=dblarr(nx)
rf=dblarr(nx)

if(ss eq 0) then nn=nn-1   
for ix=0,nx-1 do begin
   readf,lun,d1
   th(ix)=d1
endfor
for iz=0,nz-1 do begin
   readf,lun,d2
   r(iz)=d2/radconv
endfor
close,lun
   
err=0
errold=0

;--------------------------------------------------
; Read solution - HYDROEQ.DAT
;--------------------------------------------------
   
openr,lun,'Hydroeq.dat'
readf,lun,nx,nz,omg
rhon=dblarr(nz,nx)  ;density
pren=dblarr(nz,nx)  ;pressure
psin=dblarr(nz,nx)  ;conformal factor
vphn=dblarr(nz,nx)  ;phi component of velocity
alfn=dblarr(nz,nx)  ;lapse 
betn=dblarr(nz,nx)  ;phi component of shift vector
gpp=dblarr(nz,nx)   ;g(phi,phi)
gtt=dblarr(nz,nx)   ;g(theta,theta)
grr=dblarr(nz,nx)   ;g(r,r)
zzz=dblarr(nz,nx)
xxx=dblarr(nz,nx)
for ix=0,nx-1 do begin
   for iz=0,nz-1 do begin
      readf,lun,dat1,dat2,dat3,dat4,dat5,dat6
      rhon(iz,ix)=dat1
      pren(iz,ix)=dat2
      psin(iz,ix)=dat3
      vphn(iz,ix)=dat4
      alfn(iz,ix)=dat5
      betn(iz,ix)=dat6
      gpp(iz,ix)=r(iz)^2*sin(th(ix))^2*psin(iz,ix)^4*(radconv)^2
      gtt(iz,ix)=r(iz)^2*psin(iz,ix)^4*(radconv)^2
      grr(iz,ix)=psin(iz,ix)^4
      zzz(iz,ix)=r(iz)
      xxx(iz,ix)=th(ix)
   endfor
endfor
close,lun
r2d=zzz
t2d=xxx

wsurf=intarr(nx)
for ix=0,nx-1 do begin
   wsurf(ix)=min(where(alog10(pren(*,ix)) le -14. ))
endfor


;--------------------------------------------------
; Read solution - HYDROEQ_MAG.DAT
;--------------------------------------------------

openr,lun,'Hydroeq_mag.dat',ERROR=err
   
if (err eq 0) then begin
   readf,lun,nx,nz
   bmrr=dblarr(nz,nx)  ;r-component of the magnetic field
   bmtt=dblarr(nz,nx)  ;theta-component of the magnetic field
   bphn=dblarr(nz,nx)  ;phi-component of magnetic field
   aphi=dblarr(nz,nx)  ;vector potential/r sin(th)
   elrr=dblarr(nz,nx)  ;r-component of the electric field (covariant)
   eltt=dblarr(nz,nx)  ;theta-component of the electric field (covariant)
   ephn=dblarr(nz,nx)  ;phi-component of electric field (covariant)
   atim=dblarr(nz,nx)  ;electric potential
   Jphn=dblarr(nz,nx)  ;phi-component current density
   Jttn=dblarr(nz,nx)  ;th-component current density
   Jrrn=dblarr(nz,nx)  ;r-component current density
   
   for ix=0,nx-1 do begin
      for iz=0,nz-1 do begin
         readf,lun,dat1,dat2,dat3,dat4,dat5,dat6,dat7,dat8,dat9,dat10,dat11
         bphn(iz,ix)=dat1      
         bmrr(iz,ix)=dat2
         bmtt(iz,ix)=dat3      
         aphi(iz,ix)=dat4
         ephn(iz,ix)=dat5
         elrr(iz,ix)=dat6
         eltt(iz,ix)=dat7
         atim(iz,ix)=dat8
         Jphn(iz,ix)=dat9
         Jrrn(iz,ix)=dat10
         jttn(iz,ix)=dat11
      endfor
   endfor
   
endif else begin
   
   bmrr=dblarr(nz,nx)  
   bmtt=dblarr(nz,nx)  
   bphn=dblarr(nz,nx)  
   aphi=dblarr(nz,nx)  
   emrr=dblarr(nz,nx)  
   emtt=dblarr(nz,nx)  
   ephn=dblarr(nz,nx)  
   atim=dblarr(nz,nx) 
   Jphn=dblarr(nz,nx)  
   Jttn=dblarr(nz,nx)  
   Jrrn=dblarr(nz,nx)  
   
   bmrr(*,*)=0.  
   bmtt(*,*)=0.  
   bphn(*,*)=0. 
   aphi(*,*)=0.  
   emrr(*,*)=0.  
   emtt(*,*)=0.  
   ephn(*,*)=0.  
   atim(*,*)=0.
   Jphn(*,*)=0.  
   Jttn(*,*)=0. 
   Jrrn(*,*)=0.
   
endelse
close,lun

;--------------------------------------------------
; Read solution - MXWLLTEST.DAT
;--------------------------------------------------
   
openr,lun,'Mxwll_test.dat',ERROR=err

if (err eq 0) then begin
   readf,lun,nx,nz,kbpol
   rhoe=dblarr(nz,nx)
   Jphi=dblarr(nz,nx)
   atimin=dblarr(nz,nx)
   atimout=dblarr(nz,nx)
   atimarm=dblarr(nz,nx)
   omgmet=dblarr(nz,nx)
   gamloc=dblarr(nz,nx)
   drgal=dblarr(nz,nx)
   dtgal=dblarr(nz,nx)

   for ix=0,nx-1 do begin
      for iz=0,nz-1 do begin
         readf,lun,dat1,dat2,dat3,dat4,dat5,dat6,dat7,dat8,dat9
         rhoe(iz,ix)=dat1
         Jphi(iz,ix)=dat2
         atimin(iz,ix)=dat3
         atimout(iz,ix)=dat4
         atimarm(iz,ix)=dat5
         omgmet(iz,ix)=dat6
         gamloc(iz,ix)=dat7
         drgal(iz,ix)=dat8
         dtgal(iz,ix)=dat9
      endfor
   endfor

endif else begin

   kbpol=0.
   rhoe=dblarr(nz,nx)
   Jphi=dblarr(nz,nx)
   atimin=dblarr(nz,nx)
   atimout=dblarr(nz,nx)
   atimarm=dblarr(nz,nx)
   omgmet=dblarr(nz,nx)
   gamloc=dblarr(nz,nx)
   drgal=dblarr(nz,nx)
   dtgal=dblarr(nz,nx)

    rhoe(*,*)=0.
    Jphi(*,*)=0.
    atimin(*,*)=0.
    atimout(*,*)=0.
    atimarm(*,*)=0.
    omgmet(*,*)=0.
    gamloc(*,*)=0.
    drgal(*,*)=0.
    dtgal(*,*)=0.

endelse   
close,lun

glf=1./sqrt(1.-vphn^2*gpp)
gamloc=glf

v2=vphn^2*gpp
b2=bphn^2*gpp
e2pol=(elrr)^2/grr+(eltt)^2/gtt

if (output eq 1) then begin
   if(field eq 1)then filename='Density_Lin'
   if(field eq 2)then filename='Density_Log'
   if(field eq 3)then filename='Vphi_Contr'
   if(field eq 4)then filename='Vsquared'
   if(field eq 5)then filename='Lorentz'
   if(field eq 6)then filename='Btor_Lin'
   if(field eq 7)then filename='Btor_Log'
   if(field eq 8)then filename='Magnetization'
   if(field eq 9)then filename='Lapse'
   if(field eq 10)then filename='ConformalFact'
   if(field eq 11)then filename='Shift'
   if(field eq 12)then filename='Vect-pot'
   if(field eq 13)then filename='Vect-pot(rho)'
   if(field eq 14)then filename='Bpol_Lin'
   if(field eq 15)then filename='Vect-pol-tor'
   if(field eq 16)then filename='Electricf'
   if(field eq 17)then filename='Bpol-Epol'
   
   ;set_plot,'ps'
   ;device,file=filename+'.eps',/encaps,bits_per_pixel=8, /color, $
   ;xsize=16,ysize=16,xoffset=2.5,yoffset=2.5
   ;endif  

   set_plot,'ps'
   !p.font=0
   device,SCALE_FACTOR=2,file=filename+'.eps',/encaps,bits_per_pixel=8, /color, $
          xsize=16,ysize=16,xoffset=2.5,yoffset=2.5,/helvetica
   !p.thick=4
endif

;-----------------------------------------------------------------------
;READING ALL TEST FILES
;-----------------------------------------------------------------------
      
   ;TOVini.dat-------------------------------------------------------------

if(field eq 0) then begin
   if(ntest eq 1 ) then begin
      openr,lun,'TOVini.dat',ERROR=err
      
      if(err eq 0) then begin      
         readf,lun,nztov,rscwtov
         nztov=fix(nztov)
         rtov=dblarr(nztov)
         mutov=dblarr(nztov)
         rhotov=dblarr(nztov)
         nutov=dblarr(nztov)  
         prtov=dblarr(nztov)
         entov=dblarr(nztov)
         
         for iz=0,nztov-1 do begin
            readf,lun,dat1,dat2,dat3,dat4,dat5,dat6
            rtov(iz)=dat1
            mutov(iz)=dat2
            rhotov(iz)=dat3
            nutov(iz)=dat4
            prtov(iz)=dat5
            entov(iz)=dat6
         endfor
            
         close,lun
         
         print,' '
         print,'TOVini.dat has been read: '
         print,'rtov, rhotov, prtov,entov,mutov,nutov loaded.'
         
      endif else begin
         print,'TOVini.dat not found.'
      endelse

      ;Source.dat----------------------------------------------------
         
      openr,lun,'Source.dat',ERROR=err
      
      if(err eq 0) then begin
         readf,lun,nx,nz
         rhosrc=dblarr(nz,nx)
         psrc=dblarr(nz,nx)
         esrc=dblarr(nz,nx)
         
         for ix=0,nx-1 do begin
            for iz=0,nz-1 do begin
               readf,lun,dat1,dat2,dat3
               rhosrc(iz,ix)=dat1
               psrc(iz,ix)=dat2
               esrc(iz,ix)=dat3
            endfor
         endfor
         
         close,lun
         
         print,' '
         print,'Source.dat has been read: '
         print,'rhosrc, psrc, esrc loaded.'
         
      endif else begin
         print,'Source.dat not found.'
      endelse


       ;Xshifptphi.dat-----------------------------------------------

           openr,lun,'XShiftphi.dat',ERROR=err

           if(err eq 0) then begin
              readf,lun,nx,nz
              xpss=dblarr(nz,nx)
              es3src=dblarr(nz,nx)
              
              for ix=0,nx-1 do begin
                 for iz=0,nz-1 do begin
                    readf,lun,dat1,dat2
                    xpss(iz,ix)=dat1
                    es3src(iz,ix)=dat2
                 endfor
              endfor
              
              close,lun

              print,' '
              print,'XShiftphi.dat has been read: '
              print,'xpss, es3src loaded.'
              
           endif else begin

              print,'XShiftphi.dat not found.'

           endelse

            ;Conformal.dat ---------------------------------------

           openr,lun,'Conformal.dat',ERROR=err

           if(err eq 0) then begin
              readf,lun,nx,nz
              cpsi=dblarr(nz,nx)
              cecsrc=dblarr(nz,nx)
              curv=dblarr(nz,nx)
              
              for ix=0,nx-1 do begin
                 for iz=0,nz-1 do begin
                    readf,lun,dat1,dat2,dat3
                    cpsi(iz,ix)=dat1
                    cecsrc(iz,ix)=dat2
                    curv(iz,ix)=dat3
                 endfor
              endfor
              
              close,lun

              print,' '
              print,'Conformal.dat has been read: '
              print,'cpsi,cecsrc, curv loaded.'
              
           endif else begin

              print,'Conformal.dat not found.'

           endelse


           ; primitive_mag.DAT-----------------------------------
   
           openr,lun,'Primitive_Mag.dat',ERROR=err

           if(err eq 0) then begin
              readf,lun,nx,nz
              bphip=dblarr(nz,nx)
              bpolrp=dblarr(nz,nx)
              bpoltp=dblarr(nz,nx) 
              
              for ix=0,nx-1 do begin
                 for iz=0,nz-1 do begin
                    readf,lun,dat1,dat2,dat3
                    bphip(iz,ix)=dat1
                    bpolrp(iz,ix)=dat2
                    bpoltp(iz,ix)=dat3
                 endfor
              endfor
              
              close,lun

              print,' '
              print,'Primitive_Mag.dat has been read: '
              print,'bphip, bpolrp, bpoltp, loaded.'
   
           endif else begin

              print,'Primitive_Mag.dat not found.'
              
           endelse
                                
          ; LAPSE.DAT-----------------------------------------
           
           openr,lun,'Lapse.dat',ERROR=err
           
           if(err eq 0) then begin
              readf,lun,nx,nz
              psl=dblarr(nz,nx)
              elsrc=dblarr(nz,nx)
              
              for ix=0,nx-1 do begin
                 for iz=0,nz-1 do begin
                    readf,lun,dat1,dat2
                    psl(iz,ix)=dat1
                    elsrc(iz,ix)=dat2
                 endfor
              endfor
              
              close,lun

              print,' '
              print,'Lapse.dat has been read: '
              print,'psl, elsrc loaded.'
   
           endif else begin

              print,'Lapse.dat not found.'

           endelse

            ; SHIFTPHI.DAT--------------------------------------
   
           openr,lun,'Shiftphi.dat',ERROR=err
           
           if(err eq 0) then begin
              readf,lun,nx,nz
              pss=dblarr(nz,nx)
              curvp=dblarr(nz,nx)
              
              for ix=0,nx-1 do begin
                 for iz=0,nz-1 do begin
                    readf,lun,dat1,dat2
                    pss(iz,ix)=dat1
                    curvp(iz,ix)=dat2
                 endfor
              endfor
              
              close,lun

              print,' '
              print,'Shiftphi.dat has been read: '
              print,'pss, curvp loaded.'
              
           endif else begin

              print,'Shiftphi.dat not found.'
              
           endelse
      
        endif
endif


;-----------------------------------------------------------------------
;TEST FOR SOLUTIONS OF MAXWELL EQUATION
;(only for rotating stars with poloidal magnetic fields)
;-----------------------------------------------------------------------

;check convergence of central density - OK for all models
;check convergence of Aphi - Ok for all models with poloidal magnetic field
;check convergence of Atim - OK for all rotating NS with Pol. mag. field 
if(field eq 0) then begin
      if(ntest eq 1 or ntest eq 2 or ntest gt maxntest) then begin
           openr,lun,'Rhovec.dat',ERROR=err
           if(err eq 0) then begin
              nrhov=file_lines('Rhovec.dat')
              rhocnv=dblarr(nrhov)
              for ix=0,nrhov-1 do begin
                 readf,lun,dat1,dat2
                 rhocnv(ix)=dat2
              endfor
              close,lun
              print,' '
              print,'Rhovec.dat has been read: '
              print,'rhocnv loaded.'
              if(ntest eq 2 or ntest gt maxntest) then begin
                 p=plot(rhocnv,'-b',TITLE='Covergence of $rho_{c}$')
              endif
           endif else begin
              print,'Rhovec.dat not found.'
           endelse

           openr,lun,'Apconv.dat',ERROR=err
           if(err eq 0) then begin
              nap=file_lines('Apconv.dat')
              apcnv=dblarr(nap)
              for ix=0,nap-1 do begin
                 readf,lun,dat1,dat2,dat3,dat4
                 apcnv(ix)=dat2
              endfor
              close,lun
              print,' '
              print,'Apconv.dat has been read: '
              print,'apcnv loaded.'
              if(ntest eq 2 or ntest gt maxntest) then begin
                 p=plot(apcnv,'-r',TITLE='Covergence on  max $A_{\phi}$')
              endif
           endif else begin
              print,'Apconv.dat not found.'
           endelse
           
           openr,lun,'Atconv.dat',ERROR=err
           if(err eq 0) then begin
              nat=file_lines('Atconv.dat')
              atcnv=dblarr(nat)
              for ix=0,nat-1 do begin
                 readf,lun,dat1,dat2,dat3,dat4
                 atcnv(ix)=dat2
              endfor
              close,lun
              print,' '
              print,'Atconv.dat has been read: '
              print,'atcnv loaded.'
               if(ntest eq 2 or ntest gt maxntest) then begin
                 p=plot(atcnv,'-g',TITLE='Covergence on max $A_{t}$')
              endif
           endif else begin
              print,'Atconv.dat not found.'
           endelse

        endif 
   endif

;TEST Eulero / Bernoulli ----------------------------------------------

if(field eq 0) then begin
      if(ntest eq 1 or ntest eq 3 or ntest gt maxntest) then begin
         
         term1r=dblarr(nz,nx)
         term2r=dblarr(nz,nx)
         term3r=dblarr(nz,nx)
         term4r=dblarr(nz,nx)
         term4rtmp=dblarr(nz,nx)
         term1t=dblarr(nz,nx)
         term2t=dblarr(nz,nx)
         term3t=dblarr(nz,nx)
         term4t=dblarr(nz,nx)
         term4ttmp=dblarr(nz,nx)
         testBer=dblarr(nz,nx)
         
         for ix=0,nx-1 do begin
            term1r(*,ix)=DERIV(r,ALOG(1+gamma/(gamma-1.)*pren(*,ix)/rhon(*,ix)))
            term2r(*,ix)=DERIV(r,ALOG(alfn(*,ix)))
            term3r(*,ix)=-DERIV(r,ALOG(gamloc(*,ix)))
            term4rtmp(*,ix)=DERIV(r,aphi(*,ix)*r(*)*sin(th(ix)))
            ;term1r(0:wsurf(ix)-1,ix)=DERIV(r(0:wsurf(ix)-1),ALOG(1+gamma/(gamma-1.)*pren(0:wsurf(ix)-1,ix)/rhon(0:wsurf(ix)-1,ix)))
            ;term1r(wsurf(ix):nz-1,ix)=DERIV(r(wsurf(ix):nz-1),ALOG(1+gamma/(gamma-1.)*pren(wsurf(ix):nz-1,ix)/rhon(wsurf(ix):nz-1,ix)))
            ;term2r(*,ix)=DERIV(r,ALOG(alfn(*,ix)))
            ;term3r(0:wsurf(ix)-1,ix)=-DERIV(r(0:wsurf(ix)-1),ALOG(gamloc(0:wsurf(ix)-1,ix)))
            term3r(wsurf(ix):nz-1,ix)=-DERIV(r(wsurf(ix):nz-1),ALOG(gamloc(wsurf(ix):nz-1,ix)))
            ;term3r(*,ix)=-DERIV(r,ALOG(gamloc(*,ix)))
            ;term4rtmp(*,ix)=DERIV(r,aphi(*,ix)*r(*)*sin(th(ix)))
         endfor
     
         term4r=(JPHI+(omgmet-omg)/alfn*rhoe)*term4rtmp
         
         for iz=0,nz-1 do begin
            term1t(iz,*)=DERIV(th,ALOG(1+gamma/(gamma-1.)*pren(iz,*)/rhon(iz,*)))
            term2t(iz,*)=DERIV(th,ALOG(alfn(iz,*)))
            term3t(iz,*)=-DERIV(th,ALOG(gamloc(iz,*)))
            term4ttmp(iz,*)=DERIV(th,aphi(iz,*)*r(iz)*sin(th(*)))
         endfor
         term4t=(JPHI+(omgmet-omg)/alfn*rhoe)*term4ttmp

         testEr=(rhon+gamma/(gamma-1.)*pren)*(term1r+term2r+term3r)-term4r
         testEt=(rhon+gamma/(gamma-1.)*pren)*(term1t+term2t+term3t)-term4t

         enthal=(1+gamma/(gamma-1)*pren/rhon)
         TestBer=alog(enthal)+alog(alfn)-alog(gamloc)-kbpol*Aphi*r2d*sin(t2d)
         
         if(ntest eq 0) then begin
             print,' '
             print,'Test Euler equation:'
             print,'arrays TESTEr, TERM4r, TESTEt and TERM4t loaded'
         endif
         if(ntest eq 3 or ntest gt maxntest) then begin
             print,' '
             print,'Test Euler equation:'
             print,'compare array TESTEr with TERM4r and array TESTEt with TERM4t'
             
            s=surface(TESTER(0:wsurf(nx/2)+20,*),title='$r$ Euler equation')
            s=surface(TERM4r(0:wsurf(nx/2)+20,*),title='$r$ Euler equation source')
            s=surface(TESTET(0:wsurf(nx/2)+20,*),color='red',title='$\theta$ Euler equation')
            s=surface(TERM4t(0:wsurf(nx/2)+20,*),color='red',title='$\theta$ Euler equation source')

            print,'Test Bernoulli Integral:'
            print,'compare array TESTBer with central value=',alog(enthal(0,0))+alog(alfn(0,0))
            s=surface(TestBer(0:wsurf(0)+20,*),color='blue',title='Bernoulli integral (central value)')
         endif
         
      endif
endif

;TEST VECTOR MAXWELL EQUATION------------------------------------------

if(field eq 0) then begin
   if(ntest eq 1 or ntest eq 4 or ntest gt maxntest) then begin

      dterm1=dblarr(nz,nx)
      dterm2=dblarr(nz,nx)
      dter1=dblarr(nz,nx)
      dter2=dblarr(nz,nx)
      dter3=dblarr(nz,nx)
      dter4=dblarr(nz,nx)
         
      term1=psin^6*r2d^2*sin(t2d)*elrr/grr
      term2=psin^6*r2d^2*sin(t2d)*eltt/gtt
      term3=psin^6*r2d^2*sin(t2d)*rhoe
      
      for ix=0,nx-1 do begin
         ffr=term1(*,ix)
         dterm1(*,ix)=DERIV(r,ffr)
      endfor
      for iz=0,nz-1 do begin
         ffth=term2(iz,*)
         dterm2(iz,*)=DERIV(th,ffth)
      endfor
         
      testGV=dterm1+dterm2-term3
      
      ;ter1=eltt*bphn*gpp
      ;ter2=-elrr*bphn*gpp
       ter1=psin^6*r2d^2*sin(t2d)*elrr/grr*omgmet
       ter2=psin^6*r2d^2*sin(t2d)*eltt/gtt*omgmet
      ;ter3=psin^12*r2d^4*sin(t2d)^2*Bmtt/grr/gpp*alfn
      ;ter4=psin^12*r2d^4*sin(t2d)^2*Bmrr/gtt/gpp*alfn
       ter3=alfn*gtt^2*Bmtt
       ter4=alfn*grr^2*Bmrr
       ter5=-(alfn*Jphi+omgmet*rhoe)*psin^6*r2d^2*sin(t2d)
      
       for ix=0,nx-1 do begin
          ffr=ter1(*,ix)
          dter1(*,ix)=DERIV(r,ffr)
       endfor
         for iz=0,nz-1 do begin
            ffth=ter2(iz,*)
            dter2(iz,*)=DERIV(th,ffth)
         endfor
         for ix=0,nx-1 do begin
            ffr=ter3(*,ix)
            dter3(*,ix)=DERIV(r,ffr)
         endfor
         for iz=0,nz-1 do begin
            ffth=ter4(iz,*)
            dter4(iz,*)=DERIV(th,ffth)
         endfor

         testAVp=dter4-dter3-dter1-dter2-ter5
         ;testAVp=dter1+dter2+dter3-dter4-ter5


         if(ntest eq 0) then begin
            print,' '
            print,'Test Vectorial Maxwell:'
            print,'arrays testGV, ter3, testAVp, ter5 loaded'
            print,'arrays Elrr,Eltt,E2pol loaded'
         endif
         if(ntest eq 4 or ntest gt maxntest) then begin
            print,' '
            print,'Test Vectorial Maxwell equation:'
            print,'compare array testGV with term3 and array testAVp with ter5'
            print,'arrays Elrr,Eltt,E2pol loaded'
            
            s=surface(TESTGV(0:wsurf(nx/2)+20,*),title='Gauss equation (Vector)')
            s=surface(TERM3(0:wsurf(nx/2)+20,*),title='Gauss equation source')
            s=surface(TESTAVp(0:wsurf(nx/2)+20,*),color='orange',title='$\phi$-component Ampere-eq')
            s=surface(TER5(0:wsurf(nx/2)+20,*),color='orange',title='$\phi$-component Ampere-eq source')
            s=surface(Elrr(0:wsurf(nx/2)+20,*),color='red',title='r-component of E')
            s=surface(Eltt(0:wsurf(nx/2)+20,*),color='green',title='$\theta$-component of E')
            s=surface(E2pol(0:wsurf(nx/2)+20,*),color='blue',title='|E|')
         endif
                                ;Bz=Bmrr*cos(th(t2d))+
                                ;rhoe2= r2d^2*sin(t2d)*Jphi- 2*Bz
      endif
   endif

;TEST POTENTIAL MAXWELL EQUATION --------------------------------------

if(field eq 0) then begin
      if(ntest eq 1 or ntest eq 5 or ntest gt maxntest) then begin
         
         term1=dblarr(nz,nx)
         term2=dblarr(nz,nx)
         term3=dblarr(nz,nx)
         dterm1=dblarr(nz,nx)
         dterm2=dblarr(nz,nx)
         dter1=dblarr(nz,nx)
         dter2=dblarr(nz,nx)
         dter3=dblarr(nz,nx)
         dter4=dblarr(nz,nx)
         
         dApdr=dblarr(nz,nx)
         dApdr2=dblarr(nz,nx)
         dApdt=dblarr(nz,nx)
         dApdt2=dblarr(nz,nx)
         
         dAtdr=dblarr(nz,nx)
         dAtdr2=dblarr(nz,nx)
         dAtdt=dblarr(nz,nx)
         dAtdt2=dblarr(nz,nx)
         dAtdrin=dblarr(nz,nx)
         dAtdr2in=dblarr(nz,nx)
         dAtdtin=dblarr(nz,nx)
         dAtdt2in=dblarr(nz,nx)
         dAtdrout=dblarr(nz,nx)
         dAtdr2out=dblarr(nz,nx)
         dAtdtout=dblarr(nz,nx)
         dAtdt2out=dblarr(nz,nx)
         
         domgdr=dblarr(nz,nx)
         domgdt=dblarr(nz,nx)
         dmetdr=dblarr(nz,nx)
         dmetdt=dblarr(nz,nx)
         
         
         ;DERIVATE----------------------------------------------
      
         for ix=0,nx-1 do begin
            ffr=Aphi(*,ix)*r(*)*sin(th(ix))
            dApdr(*,ix)=DERIV(r,ffr)
            ffr=dApdr(*,ix)
            dApdr2(*,ix)=DERIV(r,ffr)
            ;dApdr2(0:wsurf(ix)-1,ix)=DERIV(r(0:wsurf(ix)-1),ffr(0:wsurf(ix)-1))
            ;dApdr2(wsurf(ix):nz-1,ix)=DERIV(r(wsurf(ix):nz-1),ffr(wsurf(ix):nz-1))
            ffr=Omgmet(*,ix)
            domgdr(*,ix)=DERIV(r,ffr)
            ffr=alog(psin(*,ix)^2/alfn(*,ix))
            dmetdr(*,ix)=DERIV(r,ffr)
         endfor
         for iz=0,nz-1 do begin
            ffth=Aphi(iz,*)*r(iz)*sin(th(*))
            dApdt(iz,*)=DERIV(th,ffth)
            ffth=dApdt(iz,*)
            dApdt2(iz,*)=DERIV(th,ffth)
            ffth=Omgmet(iz,*)
            domgdt(iz,*)=DERIV(th,ffth)
            ffth=alog(psin(iz,*)^2/alfn(iz,*))
            dmetdt(iz,*)=DERIV(th,ffth)
         endfor
         
         for ix=0,nx-1 do begin
            ffr=Atimin(*,ix)
            dAtdrin(*,ix)=DERIV(r,ffr)
            ffr=Atimout(*,ix)
            dAtdrout(*,ix)=DERIV(r,ffr)
            dAtdr(0:wsurf(ix)-1,ix)=dAtdrin(0:wsurf(ix)-1,ix)
            dAtdr(wsurf(ix):nz-1,ix)=dAtdrout(wsurf(ix):nz-1,ix)
         endfor

         for iz=0,nz-1 do begin
            ffth=Atimin(iz,*)
            dAtdtin(iz,*)=DERIV(th,ffth)
            ffth=Atimout(iz,*)
            dAtdtout(iz,*)=DERIV(th,ffth)
         endfor
         for ix=0,nx-1 do begin
            dAtdt(0:wsurf(ix)-1,ix)=dAtdtin(0:wsurf(ix)-1,ix)
            dAtdt(wsurf(ix):nz-1,ix)=dAtdtout(wsurf(ix):nz-1,ix)
         endfor
         
         for ix=0,nx-1 do begin
            ffr=dAtdrin(*,ix)*r(*)^2*sin(th(ix))
            dAtdr2in(*,ix)=DERIV(r,ffr)
            ffr=dAtdrout(*,ix)*r(*)^2*sin(th(ix))
            dAtdr2out(*,ix)=DERIV(r,ffr)
            dAtdr2(0:wsurf(ix)-1,ix)=dAtdr2in(0:wsurf(ix)-1,ix)
            dAtdr2(wsurf(ix):nz-1,ix)=dAtdr2out(wsurf(ix):nz-1,ix)
            ;ffr=dAtdrin(*,ix)*r(*)^2*sin(th(ix))
            ;dAtdr2in(0:wsurf(ix)-1,ix)=DERIV(r(0:wsurf(ix)-1),ffr(0:wsurf(ix)-1))
            ;ffr=dAtdrout(*,ix)*r(*)^2*sin(th(ix))
            ;dAtdr2out(wsurf(ix):nz-1,ix)=DERIV(r(wsurf(ix):nz-1),ffr(wsurf(ix):nz-1))
            ;dAtdr2(0:wsurf(ix)-1,ix)=dAtdr2in(0:wsurf(ix)-1,ix)
            ;dAtdr2(wsurf(ix):nz-1,ix)=dAtdr2out(wsurf(ix):nz-1,ix)
         endfor
         
         for iz=0,nz-1 do begin
            ffth=dAtdtin(iz,*)*sin(th(*))
            dAtdt2in(iz,*)=DERIV(th,ffth)
            ffth=dAtdtout(iz,*)*sin(th(*))
            dAtdt2out(iz,*)=DERIV(th,ffth)
         endfor
         for ix=0,nx-1 do begin
            dAtdt2(0:wsurf(ix)-1,ix)=dAtdt2in(0:wsurf(ix)-1,ix)
            dAtdt2(wsurf(ix):nz-1,ix)=dAtdt2out(wsurf(ix):nz-1,ix)
         endfor
         
         ;lapstar=dApdr2+1./r2d^2*dApdt2-1./r2d^2/tan(t2d)*dApdt
         lapstar=r2d^2*sin(t2d)*dApdr2+sin(t2d)*dApdt2-cos(t2d)*dApdt
         lap=dAtdr2+dAtdt2      ; a meno di r^2 sin th
      
         ;SOURCE TERMS -------------------------------------------
      
         sourceA1=-psin^8.*r2d^2*sin(t2d)^2.*Jphi
         domdat=domgdr*dAtdr+1/r2d^2*domgdt*dAtdt
         domdap=domgdr*dApdr+1/r2d^2*domgdt*dApdt
         sourceA2= psin^4*r2d^2*sin(t2d)^2/alfn^2*(domdat+domdap*omgmet)
         dmetdap=dmetdr*dApdr+1/r2d^2*dmetdt*dApdt
         sourceA3= dmetdap
         sourceA = sourceA1 + sourceA2+sourceA3 


         sourceB1=psin^4*(alfn*rhoe + omgmet*psin^4*r2d^2*sin(t2d)^2.*Jphi)
         sourceB2=-omgmet*psin^4*r2d^2*sin(t2d)^2./alfn^2.*domdat
         sourceB3=-(1.+omgmet^2*psin^4*r2d^2*sin(t2d)^2./alfn^2.)*domdap
         sourceB4=-dmetdr*(dAtdr+2*omgmet*dApdr)-dmetdt/r2d^2*(dAtdt+2*omgmet*dApdt)
         sourceB5=-2*omgmet/r2d*(dapdr+dapdt/r2d/tan(t2d))
         
         sourceB=sourceB1+sourceB2+sourceB3+sourceB4+sourceB5

         
         ;testA= lapstar - sourceA
         testA= lapstar - r2d^2*sin(t2d)*sourceA
         testB= lap - sourceB*r2d^2*sin(t2d)
         tMHD= OMG*Aphi*r2d*sin(t2d) + Atim


         if(ntest eq 0) then begin
            print,' '
            print,'Test Potential Maxwell equation:'
            print,'arrays testA, sourceA and lapstar for the Ampere equation loaded'
            print,'arrays testB, sourceB and lap for the Gauss equation loaded'
         endif
         if(ntest eq 5 or ntest gt maxntest) then begin
            print,' '
            print,'Test potential Maxwell equation:'
            print,'compare array testA with sourceA (Laplace) and array testB with sourceB(Gauss)'

            sourceA=r2d^2*sin(t2d)*sourceA
            sourceB=r2d^2*sin(t2d)*sourceB
            
            s=surface(testA(0:wsurf(nx/2)+50,*),title='Ampere equation (potential)')
            s=surface(sourceA(0:wsurf(nx/2)+50,*),title='Ampere equation (potential) source')
            s=surface(testB(0:wsurf(nx/2)+50,*),color='orange',title='Gauss equation (potential)')
            s=surface(sourceB(0:wsurf(nx/2)+50,*),color='orange',title='Gauss equation (potential) source')
         endif
      endif
   endif

;TEST MHD CONDITION ----------------------------------------------------

if(field eq 0) then begin
      if(ntest eq 1 or ntest eq 6 or ntest gt maxntest) then begin

         tMHD= OMG*Aphi*r2d*sin(t2d) + Atim

         testVBr=(psin^6*r2d^2*sin(t2d))*vphn*bmtt
         testVBt=-(psin^6*r2d^2*sin(t2d))*vphn*bmrr
         
         sourceint=(rhon+gamma/(gamma-1.)*pren)*kbpol
         testint=(JPHI+(omgmet-omg)/alfn*rhoe)

         if(ntest eq 0) then begin
            print,' '
            print,'Test MHD condition:'
            print,'array tMHD for the MHD condition on potential loaded'
            print,'arrays testVBr and  testVBt for the vector MHD condition loaded'
            print,'arrays testint and sourceint for the integrability condition loaded'
         endif
         if(ntest eq 6 or ntest gt maxntest) then begin
            print,' '
            print,'Test MHD condition:'
            print,'compare array tMHD with Atim (potential MHD)'
            print,'compare array testVBr with Elrr (vector MHD)'
            print,'compare array testVBt with Eltt (vector MHD)'
            print,'compare array testint with sourceint (integrability)'
            
            s=surface(tMHD,title='$\Omega A_{\phi}+A_{t}$')
            p1=plot(tMHD(0:wsurf(nx/2)+30,nx/2),'-r',name='nx|2',title='$\Omega A_{\phi}+A_{t}$')
            p2=plot(tMHD(0:wsurf(nx/4)+30,nx/4),'-b',name='nx|4',/overplot)
            p3=plot(tMHD(0:wsurf(0)+30,0),'-g',name='0',/overplot)
            l=legend(target=[p1,p2,p3])

            s=surface(testVBr(0:wsurf(nx/2)+20,*)-Elrr(0:wsurf(nx/2)+20,*),title='$-(v cross B)_r-E_r$')
            s=surface(Elrr(0:wsurf(nx/2)+20,*),title='$E_r$')
            s=surface(testVBt(0:wsurf(nx/2)+20,*)-Eltt(0:wsurf(nx/2)+20,*),$
                    color='orange',title='$-(v cross B)_{\theta}-E_{\theta}$')
            s=surface(Eltt(0:wsurf(nx/2)+20,*),title='$E_{\theta}$',color='orange')

            s=surface(testint(0:wsurf(nx/2)+20,*)-sourceint(0:wsurf(nx/2)+20,*),title='Integrability',$
                      color='green')
            s=surface(sourceint(0:wsurf(nx/2)+20,*),title='Integrability source', color='green')
         endif
         
      endif
   endif

;TEST SURFACE INTERPOLATION ---------------------------------------------

if(field eq 0 or field eq 19) then begin
  if(field eq 19) then ntest=1
   if(ntest eq 1 or ntest eq 7 or ntest gt maxntest) then begin

      openr,lun,'CheckSurf.dat',ERROR=err
      errtmp=err
      
      if(err eq 0) then begin
         readf,lun,nths,aell,bell
         rsup=dblarr(nths)
         esup=dblarr(nths)
         csup=dblarr(nths)
         asn=dblarr(nths)
         ase=dblarr(nths)
         ain=dblarr(nths)
         aout=dblarr(nths)
         for ix=0,nths-1 do begin
            readf,lun,dat1,dat2,dat3,dat4,dat5,dat6
            rsup(ix)=dat1
            esup(ix)=dat2
            asn(ix)=dat3
            ase(ix)=dat4
            ain(ix)=dat5
            aout(ix)=dat6
         endfor
         
         close,lun

         ;cassinian
         acas=sqrt((Aell^2-bell^2)/2.)
         bcas=sqrt((Aell^2+bell^2)/2.)
         for ix=1,nths-1 do begin
            cs=cos(2*th(ix))
            ss=sin(2*th(ix))
            csup(ix)=sqrt(acas^2*(-cs+sqrt((bcas/acas)^4-ss^2)))
         endfor
                  
         
         if(ntest eq 1) then begin
            print,' '
            print, 'CheckSurf.dat has been read:'
            print, 'arrays rsup, esup, asn and ase loaded'
         endif
         if(ntest eq 7 or ntest gt maxntest) then begin
            print,' '
            print,'Test surface:'
            print,'arrays rsup, esup, asn and ase loaded'
            
            p1=plot(rsup,'-b',name='num. surface',title='Surface',$
                    xtitle='$n_\theta$',ytitle='$R_{surf}$')
            p2=plot(esup,'-r',name='ellip. surface',/overplot)
            l=legend(target=[p1,p2])
            
            pp=polarplot(rsup,th-acos(-1)/2,aspect_ratio=1,title='Surface',AXIS_STYLE=2)
            pp=polarplot(esup,th-acos(-1)/2,'-r',/overplot)
            pp=polarplot(csup,th-acos(-1)/2,'-b',/overplot)
            
            p1=plot(asn,'-b',name='$At$ on num. surf.',title='Surface Potential',$
                    xtitle='$n_\theta$',ytitle='$At(R_{surf})$')
            p2=plot(ase,'-r',name='$At$ on ell. surf.',/overplot)
            l=legend(target=[p1,p2])           
         endif
         
      endif else begin
         print, 'CheckSurf.dat not found.'
      endelse
      
      openr,lun,'CheckInterpl.dat',ERROR=err
      
      if(err eq 0 and errtmp eq 0) then begin
         readf,lun,nxg,nzg
         xgqin=dblarr(nxg)
         rxqin=dblarr(nxg)
         phiin=dblarr(nxg)
         xgqout=dblarr(nxg)
         rxqout=dblarr(nxg)
         phiout=dblarr(nxg)
         
         for ix=0,nxg-1 do begin
            readf,lun,dat1,dat2,dat3,dat4,dat5,dat6
            xgqin(ix)=dat1
            rxqin(ix)=dat2
            phiin(ix)=dat3
            xgqout(ix)=dat4
            rxqout(ix)=dat5
            phiout(ix)=dat6
         endfor
         close,lun

         if(ntest eq 0) then begin
            print, ' '
            print, 'CheckInterpl.dat has been red:'
            print, 'arrays xgqin, rxqin and phiin loaded;'
            print, 'arrays xgqout, rxqout and phiout loaded.'
         endif
         if(ntest eq 7 or ntest gt maxntest) then begin
            print,' '
            print,'Test interpolation for the spectral method:'
            print, 'arrays xgqin, rxqin and phiin loaded;'
            print, 'arrays xgqout, rxqout and phiout loaded.'
            
            p1=plot(th/acos(-1),esup,'-',name='ell. surface',title='Surface Points for spectral method',$
                    xtitle='$\theta/\pi$',ytitle='$R_{surf}$')
            p2=plot(acos(xgqin(0:nxg/2))/acos(-1),rxqin,/overplot,'*r',name='inner points')
            p3=plot(acos(xgqout(0:nxg/2))/acos(-1),rxqout,/overplot,'+b',name='outer points')
            l=legend(target=[p1,p2,p3])

            
            p1=plot(th/acos(-1),ase,'-',name='$At$ on ell. surf.',title='Surface Potential for spectral method',$
                   xtitle='$\theta/\pi$',ytitle='$At(R_{surf})$')
            p2=plot(acos(xgqin(0:nxg/2))/acos(-1),phiin,/overplot,'*r',name='inner points')
            p3=plot(acos(xgqout(0:nxg/2))/acos(-1),phiout,/overplot,'+b',name='outer points')
            p3=plot(th/acos(-1),ain,/overplot,'-r',name='inner points')
            p3=plot(th/acos(-1),aout,/overplot,'-b',name='outer points')
            l=legend(target=[p1,p2,p3])           
         endif
         
      endif else begin
         print, 'CheckInterpl.dat not found.'
      endelse
      
   endif
endif

;CHECK primitive variables

if(field eq 0) then begin
      if(ntest eq 1 or ntest eq 8 or ntest gt maxntest) then begin

          ;Primitive.dat-------------------------------------------------

            openr,lun,'Primitive.dat',ERROR=err
            
            if(err eq 0) then begin
               readf,lun,nx,nz
               rhop=dblarr(nz,nx)
               prep=dblarr(nz,nx)
               enep=dblarr(nz,nx)
               vphp=dblarr(nz,nx)  
               for ix=0,nx-1 do begin
                  for iz=0,nz-1 do begin
                     readf,lun,dat1,dat2,dat3,dat4,dat5
                     rhop(iz,ix)=dat1
                     prep(iz,ix)=dat2
                     enep(iz,ix)=dat3
                     vphp(iz,ix)=dat4
                  endfor
               endfor
               close,lun
               
               if(ntest eq 0) then begin
                  print,' '
                  print,'Primitive.dat has been read: '
                  print,'rhop, prep, enep,vphp loaded.'
               endif
               if(ntest eq 8 or ntest gt maxntest) then begin
                  print,' '
                  print,'Comparison with primitives:'
                  print,'rhop, prep, enep,vphp loaded.'
                  
                  p1=plot(alog10(rhon(0:wsurf(nx/2)+20,nx/2)),'-r',name='nx|2',title='Rho')
                  p2=plot(alog10(rhon(0:wsurf(nx/4)+20,nx/4)),'-g',name='nx|4',/overplot)
                  p3=plot(alog10(rhon(0:wsurf(0)+20,0)),'-b',name='0',/overplot)
                  p4=plot(alog10(rhop(0:wsurf(nx/2)+20,nx/2)),'--2',/overplot)
                  p5=plot(alog10(rhop(0:wsurf(nx/4)+20,nx/4)),'--2',/overplot)
                  p6=plot(alog10(rhop(0:wsurf(0)+20,0)),'--2',/overplot)
                  l=legend(target=[p1,p2,p3])

                  p1=plot(alog10(pren(0:wsurf(nx/2)+20,nx/2)),'-r',name='nx|2',title='pressure')
                  p2=plot(alog10(pren(0:wsurf(nx/4)+20,nx/4)),'-g',name='nx|4',/overplot)
                  p3=plot(alog10(pren(0:wsurf(0)+20,0)),'-b',name='0',/overplot)
                  p4=plot(alog10(prep(0:wsurf(nx/2)+20,nx/2)),'--2',/overplot)
                  p5=plot(alog10(prep(0:wsurf(nx/4)+20,nx/4)),'--2',/overplot)
                  p6=plot(alog10(prep(0:wsurf(0)+20,0)),'--2',/overplot)
                  l=legend(target=[p1,p2,p3])

                  p1=plot(vphn(0:wsurf(nx/2)+20,nx/2),'-r',name='nx|2',title='velocity')
                  p2=plot(vphn(0:wsurf(nx/4)+20,nx/4),'-g',name='nx|4',/overplot)
                  p3=plot(vphn(0:wsurf(0)+20,0),'-b',name='0',/overplot)
                  p4=plot(vphp(0:wsurf(nx/2)+20,nx/2),'--2',/overplot)
                  p5=plot(vphp(0:wsurf(nx/4)+20,nx/4),'--2',/overplot)
                  p6=plot(vphp(0:wsurf(0)+20,0),'--2',/overplot)
                  l=legend(target=[p1,p2,p3])
                  
               endif   
            endif else begin
               print,'Primitive.dat not found.'
            endelse
         endif
   endif

;-----------------------------------------------------------------
;-----------------------------------------------------------------
; STANDARD VISUALIZATION 
;-----------------------------------------------------------------
;-----------------------------------------------------------------
;plot the DENSITY profile ----------------------------------------
  if(field eq 1)then begin  
   if(output eq 0) then window,0,xsize=700,ysize=700
   erase
      for ii=0,2*nxi-1 do begin
         for jj=0,2*nxi-1 do begin
            rad=sqrt(rx(ii)^2+ry(jj)^2)
            ang=acos(rx(jj)/rad)
            if( rad LE r(nn) || ss eq 0) then begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn*rad/r(nn))
            endif else begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
            endelse     
            ;iang=fix(nx*ang/3.1415)
            ;irad=fix(nz*rad/max(r))
            if(irad gt nz-1)then irad=nz-1
            arr(ii,jj)=rhon(irad,iang)*6134.
            arrcirc(ii,jj)=pren(irad,iang)
         endfor
      endfor

      amin=min(arr)
      amax=max(arr)
      arr=smooth(arr,10)

      arrcirc=smooth(arrcirc,20)
      arrcircmin=min(arrcirc)
      arrcircmax=max(arrcirc)
      
      loadct,colortable 
     
      imdisp,arr,range=[amin,amax],pos=pos1,/usepos  

      contour,arrcirc,rx,ry,levels=[pcirc],c_color=[100+0*findgen(10),255+0*FINDGEN(10),200+0*findgen(10)],$ 
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              /noerase,pos=pos1,charsize=1.5,thick=thickness
            
      alev=fltarr(100,10)
      alev0=amin+(amax-amin)*findgen(100)/99.
      for i=0,99 do alev(i,*)=alev0(i)
      imdisp,alev,pos=pos2,/usepos

     loadct,0

      contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, $
              yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5,ticklen=0.2

      contour,arr,rx,ry,nlev=6, $
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              xtitle='R(km)',ytitle='R(km)',c_color=[255+0*findgen(10),255+0*FINDGEN(10),255+0*findgen(10)], $
              /noerase, title='Density (1.e14 g/cm3)',pos=pos1,charsize=1.5,thick=thickness

   endif

;plot the LOG-DENSITY profile --------------------------------------- 
   if(field eq 2)then begin  
   if(output eq 0) then window,0,xsize=700,ysize=700
   erase
   rhonn=rhon/max(rhon)
      for ii=0,2*nxi-1 do begin
         for jj=0,2*nxi-1 do begin
            rad=sqrt(rx(ii)^2+ry(jj)^2)
            ang=acos(rx(jj)/rad)
            if( rad LE r(nn) || ss eq 0) then begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn*rad/r(nn))
            endif else begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
            endelse
            if(irad gt nz-1)then irad=nz-1
            arr(ii,jj)=alog10(rhonn(irad,iang))
         endfor
      endfor
      amin=min(arr)
      amax=max(arr)
      
      loadct,colortable 

      imdisp,arr,range=[amin,amax],pos=pos1,/usepos

      loadct,0

      contour,arr,rx,ry,nlev=5, $
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              xtitle='R(km)',ytitle='R(km)', $
              /noerase, title='Density (1.e14 g/cm3)',pos=pos1,charsize=1.5,thick=thickness
      
      loadct,colortable

      alev=fltarr(100,10)
      alev0=amin+(amax-amin)*findgen(100)/99.
      for i=0,99 do alev(i,*)=alev0(i)
      
      imdisp,alev,pos=pos2,/usepos
  
      loadct,0
      
      contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, $
              yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
   endif    

;plot the phi-component of the VELOCITY -------------------------------- 
   if(field eq 3)then begin  
   if(output eq 0) then  window,0,xsize=700,ysize=700
   erase
      for ii=0,2*nxi-1 do begin
         for jj=0,2*nxi-1 do begin
            rad=sqrt(rx(ii)^2+ry(jj)^2)
            ang=acos(rx(jj)/rad)
            if( rad LE r(nn) || ss eq 0) then begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn*rad/r(nn))
            endif else begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
            endelse
            if(irad gt nz-1)then irad=nz-1
            arr(ii,jj)=vphn(irad,iang)
         endfor
      endfor
      amin=min(arr)
      amax=max(arr)
      
      imdisp,arr,range=[amin,amax],pos=pos1,/usepos
      contour,arr,rx,ry,nlev=10, $
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              xtitle='R(km)',ytitle='R(km)', $
              /noerase, title='V^phi (units of c)',pos=pos1,charsize=1.5,thick=thickness
      
      alev=fltarr(100,10)
      alev0=amin+(amax-amin)*findgen(100)/99.
      for i=0,99 do alev(i,*)=alev0(i)
      imdisp,alev,pos=pos2,/usepos
      contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, $
              yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
      
   endif    

;plot v^2 ------------------------------------------------------------
   if(field eq 4)then begin  
   if(output eq 0) then window,0,xsize=700,ysize=700
   erase
      for ii=0,2*nxi-1 do begin
         for jj=0,2*nxi-1 do begin
            rad=sqrt(rx(ii)^2+ry(jj)^2)
            ang=acos(rx(jj)/rad)
            if( rad LE r(nn) || ss eq 0) then begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn*rad/r(nn))
            endif else begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
            endelse
            if(irad gt nz-1)then irad=nz-1
            arr(ii,jj)=v2(irad,iang)
         endfor
      endfor
      amin=min(arr)
      amax=max(arr)
      
      imdisp,arr,range=[amin,amax],pos=pos1,/usepos
      contour,arr,rx,ry,nlev=10, $
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              xtitle='R(km)',ytitle='R(km)', $
              /noerase, title='V!u2!n (units of c!u2!n)',pos=pos1,charsize=1.5,thick=thickness
      
      alev=fltarr(100,10)
      alev0=amin+(amax-amin)*findgen(100)/99.
      for i=0,99 do alev(i,*)=alev0(i)
      imdisp,alev,pos=pos2,/usepos
      contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, $
              yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
      
   endif  

;plot Lorentz Factor ---------------------------------------------------------   
   if(field eq 5)then begin  
   if(output eq 0) then window,0,xsize=700,ysize=700
   erase
      for ii=0,2*nxi-1 do begin
         for jj=0,2*nxi-1 do begin
            rad=sqrt(rx(ii)^2+ry(jj)^2)
            ang=acos(rx(jj)/rad)
            if( rad LE r(nn) || ss eq 0) then begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn*rad/r(nn))
            endif else begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
            endelse
            if(irad gt nz-1)then irad=nz-1
            arr(ii,jj)=glf(irad,iang)
         endfor
      endfor
      amin=min(arr)
      amax=max(arr)
      
      imdisp,arr,range=[amin,amax],pos=pos1,/usepos
      contour,arr,rx,ry,nlev=10, $
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              xtitle='R(km)',ytitle='R(km)', $
              /noerase, title='Lorentz factor',pos=pos1,charsize=1.5,thick=thickness
      
      alev=fltarr(100,10)
      alev0=amin+(amax-amin)*findgen(100)/99.
      for i=0,99 do alev(i,*)=alev0(i)
      imdisp,alev,pos=pos2,/usepos
      contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, $
              yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
      
   endif      

;plot Toroidal Magnetic field --------------------------------------------------
  if(field eq 6)then begin  
   if(output eq 0) then  window,0,xsize=700,ysize=700
  erase
      for ii=0,2*nxi-1 do begin
         for jj=0,2*nxi-1 do begin
            rad=sqrt(rx(ii)^2+ry(jj)^2)
            ang=acos(rx(jj)/rad)
            if( rad LE r(nn) || ss eq 0) then begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn*rad/r(nn))
            endif else begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
            endelse
            if(irad gt nz-1)then irad=nz-1
            arr(ii,jj)=sqrt(b2(irad,iang))*(83.3)
	    arrcirc(ii,jj)=pren(irad,iang)	
         endfor
      endfor

     arr=smooth(arr,5)
     arrtmp=arr
     arr=smooth(arr,10)
     arr(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)=arrtmp(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)
     amin=min(arr)
     amax=max(arr)

     arrcirc=smooth(arrcirc,10)
     arrcircmin=min(arrcirc)
     arrcircmax=max(arrcirc)

      
  loadct,colortable
      imdisp,arr,range=[amin,amax],pos=pos1,/usepos

      contour,arrcirc,rx,ry,levels=[pcirc],C_LINESTYLE=0,c_color=[100+0*findgen(10),255+0*FINDGEN(10),200+0*findgen(10)],$ 
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              /noerase,pos=pos1,charsize=1.5,thick=thickness

      alev=fltarr(100,10)
      alev0=amin+(amax-amin)*findgen(100)/99.
      for i=0,99 do alev(i,*)=alev0(i)

      imdisp,alev,pos=pos2,/usepos

  loadct,0

      contour,arr,rx,ry,nlev=5, $
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              xtitle='R(km)',ytitle='R(km)',c_color=[255+0*findgen(10),255+0*FINDGEN(10),255+0*findgen(10)], $
              /noerase, title='B (1.e18 G)',pos=pos1,charsize=1.5,thick=thickness

      contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, TICKLEN=0.2, $
              yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
      
 endif      

;plot Log Toroidal Magnetic field --------------------------------------------------
  if(field eq 7)then begin  
   if(output eq 0) then window,0,xsize=700,ysize=700
   erase
   b2mg=+max(sqrt(b2)*(83.3))/1000.
      for ii=0,2*nxi-1 do begin
         for jj=0,2*nxi-1 do begin
            rad=sqrt(rx(ii)^2+ry(jj)^2)
            ang=acos(rx(jj)/rad)
            if( rad LE r(nn) || ss eq 0) then begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn*rad/r(nn))
            endif else begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
            endelse
            if(irad gt nz-1)then irad=nz-1
            arr(ii,jj)=alog10(sqrt(b2(irad,iang))*(83.3)+b2mg)
         endfor
      endfor
      amin=min(arr)
      amax=max(arr)
      

      loadct,colortable
      imdisp,arr,range=[amin,amax],pos=pos1,/usepos

      loadct,0
      contour,arr,rx,ry,nlev=10, $
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              xtitle='R(km)',ytitle='R(km)', $
              /noerase, title='B (1.e18 G)',pos=pos1,charsize=1.5,thick=thickness
      
      alev=fltarr(100,10)
      alev0=amin+(amax-amin)*findgen(100)/99.
      for i=0,99 do alev(i,*)=alev0(i)

      loadct,colortable
      imdisp,alev,pos=pos2,/usepos

      loadct,0
      contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, $
              yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
      
   endif      

;plot magnetization --------------------------------------------------
  if(field eq 8)then begin  
   if(output eq 0) then window,0,xsize=700,ysize=700
   erase
   b2mg=+max(sqrt(b2)*(83.3))/1000.
      for ii=0,2*nxi-1 do begin
         for jj=0,2*nxi-1 do begin
            rad=sqrt(rx(ii)^2+ry(jj)^2)
            ang=acos(rx(jj)/rad)
            if( rad LE r(nn) || ss eq 0) then begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn*rad/r(nn))
            endif else begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
            endelse
            if(irad gt nz-1)then irad=nz-1
            arr(ii,jj)=b2(irad,iang)/(gam/(gam-1.)*pren(irad,iang))
         endfor
      endfor
      amin=min(arr)
      amax=max(arr)
      
      loadct,colortable
      imdisp,arr,range=[amin,amax],pos=pos1,/usepos

      loadct,0
      contour,arr,rx,ry,nlev=10, $
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              xtitle='R(km)',ytitle='R(km)', $
              /noerase, title='Magnetization',pos=pos1,charsize=1.5,thick=thickness
      
      alev=fltarr(100,10)
      alev0=amin+(amax-amin)*findgen(100)/99.
      for i=0,99 do alev(i,*)=alev0(i)

      loadct,colortable
      imdisp,alev,pos=pos2,/usepos

      loadct,0
      contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, $
              yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
      
   endif      

;plot lapse function------------------------------------------
  if(field eq 9)then begin  
   if(output eq 0) then window,0,xsize=700,ysize=700
   erase
   b2mg=+max(sqrt(b2)*(83.3))/1000.
      for ii=0,2*nxi-1 do begin
         for jj=0,2*nxi-1 do begin
            rad=sqrt(rx(ii)^2+ry(jj)^2)
            ang=acos(rx(jj)/rad)
            if( rad LE r(nn) || ss eq 0) then begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn*rad/r(nn))
            endif else begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
            endelse
            if(irad gt nz-1)then irad=nz-1
            arr(ii,jj)=alfn(irad,iang)
         endfor
      endfor
      amin=min(arr)
      amax=max(arr)
      
      loadct,colortable
      imdisp,arr,range=[amin,amax],pos=pos1,/usepos

      loadct,0
      contour,arr,rx,ry,nlev=10, $
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              xtitle='R(km)',ytitle='R(km)', $
              /noerase, title='Lapse',pos=pos1,charsize=1.5,thick=thickness
      
      alev=fltarr(100,10)
      alev0=amin+(amax-amin)*findgen(100)/99.
      for i=0,99 do alev(i,*)=alev0(i)

      loadct,colortable
      imdisp,alev,pos=pos2,/usepos

      loadct,0
      contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, $
              yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
      
   endif    

;plot conformal factor --------------------------------------
  if(field eq 10)then begin  
     if(output eq 0) then window,0,xsize=700,ysize=700
     erase
     for ii=0,2*nxi-1 do begin
        for jj=0,2*nxi-1 do begin
           rad=sqrt(rx(ii)^2+ry(jj)^2)
           ang=acos(rx(jj)/rad)
           if( rad LE r(nn) || ss eq 0) then begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn*rad/r(nn))
           endif else begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
           endelse
           if(irad gt nz-1)then irad=nz-1
           arr(ii,jj)=psin(irad,iang)
        endfor
     endfor
     amin=min(arr)
     amax=max(arr)
   
     loadct,colortable
     imdisp,arr,range=[amin,amax],pos=pos1,/usepos

     loadct,0
     contour,arr,rx,ry,nlev=10, $
             xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
             xtitle='R(km)',ytitle='R(km)', $
             /noerase, title='Conformal Factor',pos=pos1,charsize=1.5,thick=thickness
      
     alev=fltarr(100,10)
     alev0=amin+(amax-amin)*findgen(100)/99.
     for i=0,99 do alev(i,*)=alev0(i)
     
     loadct,colortable
     imdisp,alev,pos=pos2,/usepos
     
     loadct,0
     contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, $
             yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
     
  endif  

;plot phi component of shift vector---------------------------------  
  if(field eq 11)then begin  
     if(output eq 0) then window,0,xsize=700,ysize=700
     erase
     for ii=0,2*nxi-1 do begin
        for jj=0,2*nxi-1 do begin
           rad=sqrt(rx(ii)^2+ry(jj)^2)
           ang=acos(rx(jj)/rad)
           if( rad LE r(nn) || ss eq 0) then begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn*rad/r(nn))
           endif else begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
           endelse
           if(irad gt nz-1)then irad=nz-1
           arr(ii,jj)=betn(irad,iang)
        endfor
     endfor
     amin=min(arr)
     amax=max(arr)
     
     loadct,colortable
     imdisp,arr,range=[amin,amax],pos=pos1,/usepos
     
     loadct,0
     contour,arr,rx,ry,nlev=10, $
             xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
             xtitle='R(km)',ytitle='R(km)', $
             /noerase, title='Shift',pos=pos1,charsize=1.5,thick=thickness
     
     alev=fltarr(100,10)
     alev0=amin+(amax-amin)*findgen(100)/99.
     for i=0,99 do alev(i,*)=alev0(i)

     loadct,colortable
     imdisp,alev,pos=pos2,/usepos

     loadct,0
     contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, $
             yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
      
  endif
  
;plot the phi component of the vector potential---------------------- 
  if(field eq 12)then begin  
     if(output eq 0) then window,0,xsize=700,ysize=700
     erase
     for ii=0,2*nxi-1 do begin
        for jj=0,2*nxi-1 do begin
           rad=sqrt(rx(ii)^2+ry(jj)^2)
           ang=acos(rx(jj)/rad)
           if( rad LE r(nn) || ss eq 0) then begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn*rad/r(nn))
           endif else begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
           endelse
           if(irad gt nz-1)then irad=nz-1
           arr(ii,jj)=aphi(irad,iang)*6134.
        endfor
     endfor
     amin=min(arr)
     amax=max(arr)

     imdisp,arr,range=[amin,amax],pos=pos1,/usepos
     contour,arr,rx,ry,nlev=20, $
             xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
             xtitle='R(km)',ytitle='R(km)', $
             /noerase, title='Density (1.e14 g/cm3)',pos=pos1,charsize=1.5,thick=thickness
     
     alev=fltarr(100,10)
     alev0=amin+(amax-amin)*findgen(100)/99.
     for i=0,99 do alev(i,*)=alev0(i)
     imdisp,alev,pos=pos2,/usepos
     contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, $
             yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
  endif

;plot the density profile together with the isocontour of the phi-potential
  if(field eq 13)then begin  
     if(output eq 0) then window,0,xsize=700,ysize=700
     erase
     for ii=0,2*nxi-1 do begin
        for jj=0,2*nxi-1 do begin
           rad=sqrt(rx(ii)^2+ry(jj)^2)
           ang=acos(rx(jj)/rad)
           if( rad LE r(nn) || ss eq 0) then begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn*rad/r(nn))
           endif else begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
           endelse
           if(irad gt nz-1)then irad=nz-1
           arr(ii,jj)=alog10(rhon(irad,iang)*6134.)
        endfor
     endfor
     amin=min(arr)
     amax=max(arr)
     
     arr1=arr
     amin1=min(arr)
     amax1=max(arr)
     loadct, colortable
     imdisp,arr1,range=[amin1,amax1],pos=pos1,/usepos

     aphit=aphi*r2d*sin(t2d)
     for ii=0,2*nxi-1 do begin
        for jj=0,2*nxi-1 do begin
           rad=sqrt(rx(ii)^2+ry(jj)^2)
           ang=acos(rx(jj)/rad)
           if( rad LE r(nn) || ss eq 0) then begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn*rad/r(nn))
           endif else begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
           endelse
           if(irad gt nz-1)then irad=nz-1
           arr(ii,jj)=aphit(irad,iang)*6134.
        endfor
     endfor
     arr=smooth(arr,5)
     arrtmp=arr
     arr=smooth(arr,10)
     arr(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)=arrtmp(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)
     amin=min(arr)
     amax=max(arr)

     loadct,0
     contour,arr,rx,ry,nlev=20, $
             xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
             xtitle='R(km)',ytitle='R(km)', $
             /noerase, title='Density (1.e14 g/cm3)',pos=pos1,charsize=1.5,thick=thickness
     
     alev=fltarr(100,10)
     alev0=amin1+(amax1-amin1)*findgen(100)/99.
     for i=0,99 do alev(i,*)=alev0(i)
     loadct, colortable
     imdisp,alev,pos=pos2,/usepos
     loadct, 0
     contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, $
             yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
     
  endif      
 
;plot the poloidal magnetic field -----------------------------------
b2pol=bmtt^2*gtt+bmrr^2*grr
  if(field eq 14)then begin  
    if(output eq 0) then window,0,xsize=700,ysize=700
   erase

aphit=aphi*r2d*sin(t2d)
      for ii=0,2*nxi-1 do begin
         for jj=0,2*nxi-1 do begin
            rad=sqrt(rx(ii)^2+ry(jj)^2)
            ang=acos(rx(jj)/rad)
            if( rad LE r(nn) || ss eq 0) then begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn*rad/r(nn))
            endif else begin
              iang=fix(nx*ang/3.1415)
              irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
            endelse
            if(irad gt nz-1)then irad=nz-1
            arr(ii,jj)=sqrt(b2pol(irad,iang))*(83.3)
            arrcirc(ii,jj)=pren(irad,iang)
            arraphi(ii,jj)=aphit(irad,iang)*6134.
         endfor
      endfor
      
     arr=smooth(arr,10)
     amin=min(arr)
     amax=max(arr)

     arraphi=smooth(arraphi,5)
     arrtmp=arraphi
     arraphi=smooth(arraphi,10)
     arraphi(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)=arrtmp(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)
     amina=min(arraphi)
     amaxa=max(arraphi)
     
     arrcirc=smooth(arrcirc,10)
     arrcircmin=min(arrcirc)
     arrcircmax=max(arrcirc)


      loadct,colortable
      
      imdisp,arr,range=[amin,amax],pos=pos1,/usepos

      contour,arrcirc,rx,ry,levels=[pcirc],$
              c_color=[100+0*findgen(10),255+0*FINDGEN(10),200+0*findgen(10)], $
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              /noerase,pos=pos1,charsize=1.5, thick=thickness
      
      alev=fltarr(100,10)
      alev0=amin+(amax-amin)*findgen(100)/99.
      for i=0,99 do alev(i,*)=alev0(i)

      imdisp,alev,pos=pos2,/usepos

      loadct, 0

      contour,arraphi,rx,ry,nlev=10, $
             xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
             xtitle='R(km)',ytitle='R(km)',title='B (1.e18 G)', $
             /noerase,pos=pos1,charsize=1.5,c_color=[255+0*findgen(10),255+0*FINDGEN(10),255+0*findgen(10)], $
	     thick=thickness

      contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, TICKLEN=0.2, $
              yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
      
endif      

;plot for twisted torus models -----------------------------------
  pos1=[.23,.2,.83,.80]
  posA=[.23,.2,.53,.80]
  posB=[.53,.2,.83,.80]
  pos2A=[.1,.10,.13,.90]
  pos2B=[.95,.10,.98,.90]

  b2pol=bmtt^2*gtt+bmrr^2*grr

if(field eq 15)then begin  
   if(output eq 0) then window,0,xsize=700,ysize=700
   erase
   for ii=0,nxi-1 do begin
      for jj=0,2*nxi-1 do begin
         rad=sqrt(rx(ii)^2+ry(jj)^2)
         ang=acos(rx(jj)/rad)
         if( rad LE r(nn) || ss eq 0) then begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn*rad/r(nn))
         endif else begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
         endelse
         if(irad gt nz-1)then irad=nz-1
         arrA(ii,jj)=sqrt(b2(irad,iang))*(83.3)*1.e4
         rxA(ii)=rx(ii)
      endfor
   endfor
   arrA=smooth(arrA,10)
   amin=min(arrA)
   amax=max(arrA) 
   
   for ii=0,nxi-1 do begin
      for jj=0,2*nxi-1 do begin
         rad=sqrt(rx(ii+nxi)^2+ry(jj)^2)
         ang=acos(rx(jj)/rad)
         
         if( rad LE r(nn) || ss eq 0) then begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn*rad/r(nn))
         endif else begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
         endelse
         
         if(irad gt nz-1)then irad=nz-1  
            arrB(ii,jj)=sqrt(b2pol(irad,iang))*(83.3)*1.e4
            rxB(ii)=rx(ii+nxi)
         endfor
      endfor
      
   arrB=smooth(arrB,10)
   amin2=min(arrB)
   amax2=max(arrB)
   
   aphit=aphi*r2d*sin(t2d)
   for ii=0,2*nxi-1 do begin
      for jj=0,2*nxi-1 do begin
         rad=sqrt(rx(ii)^2+ry(jj)^2)
         ang=acos(rx(jj)/rad)
         
         if( rad LE r(nn) || ss eq 0) then begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn*rad/r(nn))
         endif else begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
         endelse
         if(irad gt nz-1)then irad=nz-1
         arraphi(ii,jj)=aphit(irad,iang)*6134.
         arrcirc(ii,jj)=pren(irad,iang)
      endfor
   endfor
   arraphi=smooth(arraphi,5)
   arrtmp=arraphi
   arraphi=smooth(arraphi,10)
   arraphi(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)=arrtmp(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)
   amina=min(arraphi)
   amaxa=max(arraphi)
   
   arrcirc=smooth(arrcirc,20)
   arrcircmin=min(arrcirc)
   arrcircmax=max(arrcirc)
   
   loadct,colortable
   imdisp,arrA,range=[amin,amax],pos=posA,/usepos
   imdisp,arrB,range=[amin2,amax2],pos=posB,/usepos
   
   
   contour,arrcirc,rx,ry,levels=[pcirc],$
           c_color=[100+0*findgen(10),255+0*FINDGEN(10),200+0*findgen(10)], $
           xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
           /noerase,pos=pos1,charsize=1.5, thick=thickness

   
   alev=fltarr(10,100)
   alev0=amin+(amax-amin)*findgen(100)/99.
   for i=0,99 do alev(*,i)=alev0(i)
  
   loadct,0
   contour,arraphi,rx,ry,nlev=10, $
           xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
           xtitle='R(km)', title='Btor | Bpol (1.e14 G)', $
           /noerase,pos=pos1,charsize=1.5,c_color=[255+0*findgen(10),255+0*FINDGEN(10),255+0*findgen(10)],$
           thick=thickness   
   
   loadct,colortable
   imdisp,alev,pos=pos2A,/usepos

    for i=0l,nz-2 do begin
  	if( pren(i,nx/2) gt 1.e-15) then flag=i
   endfor
   lamb=4 
   aphilev=aphi(lamb*flag,nx/2)*r2d(lamb*flag)*6134. 
   contour,arraphi,rx,ry,levels=[aphilev],$
           c_color=[500+0*findgen(10),200+0*FINDGEN(10),200+0*findgen(10)], $
           xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
           /noerase,pos=pos1,charsize=1.5, thick=1.8*thickness
   
   loadct,0
   contour,alev,indgen(10),alev0,yrange=[amin,amax],xstyle=1,ystyle=1, $
              xticks=1,xtickname=[' ',' '],/nodata,/noerase,pos=pos2A,charsize=1.5
      
   alev=fltarr(10,100)
   alev0=amin2+(amax2-amin2)*findgen(100)/99.
   for i=0,99 do alev(*,i)=alev0(i)
   loadct,colortable

   imdisp,alev,pos=pos2B,/usepos

   loadct,0
   contour,alev,indgen(10),alev0,yrange=[amin2,amax2],xstyle=1,ystyle=1, $
           xticks=1,xtickname=[' ',' '],/nodata,/noerase,pos=pos2B,charsize=1.5
      
endif 

;plot electric field -----------------------------------------
  if(field eq 16) then begin
     if(output eq 0) then window,0,xsize=700,ysize=700
     erase
     
     for ii=0,2*nxi-1 do begin
        for jj=0,2*nxi-1 do begin
           rad=sqrt(rx(ii)^2+ry(jj)^2)
           ang=acos(rx(jj)/rad)
           if( rad LE r(nn) || ss eq 0) then begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn*rad/r(nn))
           endif else begin
             iang=fix(nx*ang/3.1415)
             irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
           endelse
           if(irad gt nz-1)then irad=nz-1
           arr(ii,jj)=sqrt(e2pol(irad,iang))*8.35299e19/1.e12
           arrcirc(ii,jj)=pren(irad,iang)
           arratim(ii,jj)=atim(irad,iang)
           if(rx(ii) gt 0) then begin
              Exx(ii,jj)=Elrr(irad,iang)/sqrt(grr(irad,iang))*sin(ang)+Eltt(irad,iang)/sqrt(gtt(irad,iang))*cos(ang)
              Eyy(ii,jj)=Elrr(irad,iang)/sqrt(grr(irad,iang))*cos(ang)-Eltt(irad,iang)/sqrt(gtt(irad,iang))*sin(ang)
           endif else begin
              Exx(ii,jj)=-Elrr(irad,iang)/sqrt(grr(irad,iang))*sin(ang)-Eltt(irad,iang)/sqrt(gtt(irad,iang))*cos(ang)
              Eyy(ii,jj)=Elrr(irad,iang)/sqrt(grr(irad,iang))*cos(ang)-Eltt(irad,iang)/sqrt(gtt(irad,iang))*sin(ang)
           endelse
        endfor
     endfor
     
     
      arr=smooth(arr,3)
      amin=min(arr)
      amax=max(arr)
      
      arratim=smooth(arratim,1)
      arrtmp=arratim
      arratim=smooth(arratim,1)
      arratim(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)=arrtmp(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)
      amina=min(arratim)
      amaxa=max(arratim)
      arrcircmin=min(arrcirc)
      arrcircmax=max(arrcirc)
     
      loadct,colortable

      imdisp,arr,range=[amin,amax],pos=pos1,/usepos

      contour,arrcirc,rx,ry,levels=[pcirc],$
              c_color=[100+0*findgen(10),255+0*FINDGEN(10),200+0*findgen(10)], $
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              /noerase,pos=pos1,charsize=1.5, thick=thickness

      alev=fltarr(100,10)
      alev0=amin+(amax-amin)*findgen(100)/99.
      for i=0,99 do alev(i,*)=alev0(i)
      
      imdisp,alev,pos=pos2,/usepos
      
      loadct, 0
      
      contour,arratim,rx,ry,nlev=10, $
              xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
              xtitle='R(km)',ytitle='R(km)',title='E (1.e12 statvolt/cm)', $
              /noerase,pos=pos1,charsize=1.5,c_color=[255+0*findgen(10),255+0*FINDGEN(10),255+0*findgen(10)], $
              thick=thickness

      contour,alev,alev0,indgen(10),xrange=[amin,amax],xstyle=1,ystyle=1, TICKLEN=0.2, $
              yticks=1,ytickname=[' ',' '],/nodata,/noerase,pos=pos2,charsize=1.5
      
   endif
  

if(field eq 17)then begin  
   if(output eq 0) then window,0,xsize=700,ysize=700
   erase

   for ii=0,nxi-1 do begin
      for jj=0,2*nxi-1 do begin
         rad=sqrt(rx(ii)^2+ry(jj)^2)
         ang=acos(rx(jj)/rad)
         
         if( rad LE r(nn) || ss eq 0) then begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn*rad/r(nn))
         endif else begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
         endelse
         if(irad gt nz-1)then irad=nz-1
         arrA(ii,jj)=sqrt(e2pol(irad,iang))*8.35299d19/1.d15
         rxA(ii)=rx(ii)
      endfor
   endfor
   arrA=smooth(arrA,1)
   amin=min(arrA)
   amax=max(arrA)


   for ii=0,nxi-1 do begin
      for jj=0,2*nxi-1 do begin
         rad=sqrt(rx(ii+nxi)^2+ry(jj)^2)
         ang=acos(rx(jj)/rad)
         if( rad LE r(nn) || ss eq 0) then begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn*rad/r(nn))
         endif else begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
         endelse   
         if(irad gt nz-1)then irad=nz-1
         arrB(ii,jj)=sqrt(b2pol(irad,iang))*(83.3)
         rxB(ii)=rx(ii+nxi)
      endfor
   endfor
      
   arrB=smooth(arrB,2)
   amin2=min(arrB)
   amax2=max(arrB)

   aphit=aphi*r2d*sin(t2d)
   for ii=0,2*nxi-1 do begin
      for jj=0,2*nxi-1 do begin
         rad=sqrt(rx(ii)^2+ry(jj)^2)
         ang=acos(rx(jj)/rad)
         
         if( rad LE r(nn) || ss eq 0) then begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn*rad/r(nn))
         endif else begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
         endelse
         if(irad gt nz-1) then irad=nz-1
         arraphi(ii,jj)=aphit(irad,iang)  ;*6134.
         arratim(ii,jj)=atim(irad,iang)   ;*6134.
         arrcirc(ii,jj)=pren(irad,iang)
      endfor
   endfor
   arraphi=smooth(arraphi,2)
   arrtmp=arraphi
   arraphi=smooth(arraphi,4)
   arraphi(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)=arrtmp(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)
   amina=min(arraphi)
   amaxa=max(arraphi)

   arratim=smooth(arratim,1)
   ;arrtmp=arratim
   ;arraphi=smooth(arratim,5)
  ; arratim(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)=arrtmp(nxi/2:3./2.*nxi,nxi/2:3./2.*nxi)
   aminat=min(arratim)
   amaxat=max(arratim)
   
   arrcirc=smooth(arrcirc,3)
   arrcircmin=min(arrcirc)
   arrcircmax=max(arrcirc)
   
   loadct,colortable
   imdisp,arrA,range=[amin,amax],pos=posA,/usepos
   imdisp,arrB,range=[amin2,amax2],pos=posB,/usepos
   
       
   contour,arrcirc,rx,ry,levels=[pcirc],$
           c_color=[100+0*findgen(10),255+0*FINDGEN(10),200+0*findgen(10)], $
           xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
           /noerase,pos=pos1,charsize=1.5, thick=thickness
      
   alev=fltarr(10,100)
   alev0=amin+(amax-amin)*findgen(100)/99.
   for i=0,99 do alev(*,i)=alev0(i)
   loadct,0

      ;contour,arrA,rxA,ry,nlev=5, $
      ;        xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
      ;        xtitle='R(km)',ytitle='R(km)', $
      ;        /noerase,title='B (1.e18 G)',pos=pos1,charsize=1.5

      ;contour,arrB,rxB,ry,nlev=10, $
      ;        xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
      ;        xtitle='R(km)',ytitle='R(km)', $
      ;        /noerase,title='B (1.e18 G)',pos=pos1,charsize=1.5


   contour,arraphi(nxi:2*nxi-1,*),rx(nxi:2*nxi-1),ry,nlev=10, $
           xrange=[0.,rmax],yrange=[-rmax,rmax],xstyle=5,ystyle=5, $
           title=' Bpol (1.e18 G)', $
           /noerase,charsize=1.5,c_color=[255+0*findgen(10),255+0*FINDGEN(10),255+0*findgen(10)],$
           thick=thickness,pos=posB
   
   contour,arratim(0:nxi-1,*),rx(0:nxi-1,*),ry,nlev=10, $
           xrange=[-rmax,0.],yrange=[-rmax,rmax],xstyle=5,ystyle=5, $
           title='Epol (1.e15 sV/cm)', $
           /noerase,charsize=1.5,c_color=[255+0*findgen(10),255+0*FINDGEN(10),255+0*findgen(10)],$
           thick=thickness,pos=posA
   
   contour,arrcirc*0.,rx,ry,levels=[pcirc],$
           xrange=[-rmax,rmax],yrange=[-rmax,rmax],xstyle=1,ystyle=1, $
           xtitle='R(km)', /noerase,pos=pos1,charsize=1.5, thick=thickness
   
   

      
   loadct,colortable
   imdisp,alev,pos=pos2A,/usepos
   
   loadct,0
   contour,alev,indgen(10),alev0,yrange=[amin,amax],xstyle=1,ystyle=1, $
           xticks=1,xtickname=[' ',' '],/nodata,/noerase,pos=pos2A,charsize=1.5
   
   alev=fltarr(10,100)
   alev0=amin2+(amax2-amin2)*findgen(100)/99.
   for i=0,99 do alev(*,i)=alev0(i)
   
   loadct,colortable
   
   imdisp,alev,pos=pos2B,/usepos
   
   loadct,0
   contour,alev,indgen(10),alev0,yrange=[amin2,amax2],xstyle=1,ystyle=1, $
           xticks=1,xtickname=[' ',' '],/nodata,/noerase,pos=pos2B,charsize=1.5
   
endif 
  
  
  if(field eq 30)then begin   
     for ii=0,2*nxi-1 do begin
        for jj=0,2*nxi-1 do begin
          rad=sqrt(rx(ii)^2+ry(jj)^2)
          ang=acos(rx(jj)/rad)
          if( rad LE r(nn) || ss eq 0) then begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn*rad/r(nn))
          endif else begin
            iang=fix(nx*ang/3.1415)
            irad=fix(nn+alog(1 + nn*(rad-r(nn))*(ss-1)/(ss*r(nn)) )/alog(ss) )
          endelse
          if(irad gt nz-1)then irad=nz-1
          if(rx(ii) gt 0) then begin
          Exx(ii,jj)=Elrr(irad,iang)/sqrt(grr(irad,iang))*sin(ang)+Eltt(irad,iang)/sqrt(gtt(irad,iang))*cos(ang)
          Eyy(ii,jj)=Elrr(irad,iang)/sqrt(grr(irad,iang))*cos(ang)-Eltt(irad,iang)/sqrt(gtt(irad,iang))*sin(ang)
          arr(ii,jj)=sqrt(e2pol(irad,iang))
          endif else begin
          Exx(ii,jj)=-Elrr(irad,iang)/sqrt(grr(irad,iang))*sin(ang)-Eltt(irad,iang)/sqrt(gtt(irad,iang))*cos(ang)
          Eyy(ii,jj)=Elrr(irad,iang)/sqrt(grr(irad,iang))*cos(ang)-Eltt(irad,iang)/sqrt(gtt(irad,iang))*sin(ang)
          arr(ii,jj)=-sqrt(e2pol(irad,iang))
          endelse
          ;arr(ii,jj)=sqrt(e2pol(irad,iang))
          arrcirc(ii,jj)=pren(irad,iang)
          arratim(ii,jj)=atim(irad,iang)
        endfor
     endfor

      Elrin=rsup
      Elrout=rsup
      Eltin=rsup
      Eltout=rsup
      Brout=rsup
      Btout=rsup
      Brin=rsup
      Btin=rsup
      grrs=rsup
      gtts=rsup	
      grrsin=rsup
      gttsin=rsup
      Lfr=rsup
      Lft=rsup
      Lfb=rsup
      omgs=rsup
      alfns=rsup
      vphns=rsup
      psins=rsup
      
        
    for ii=0,nx-1 do begin
     ; rflag(ii)=max(where( abs(rhoe(*,ii)) gt 0. ))
      rf(ii)=where( r eq rsup(ii))
      fl=rf(ii)
      Elrin(ii)=INTERPOL(Elrr(0:fl,ii),r(0:fl),[(r(fl+1)+r(fl))/2.])
      Eltin(ii)=INTERPOL(Eltt(0:fl,ii),r(0:fl),[(r(fl+1)+r(fl))/2.])
      Elrout(ii)=INTERPOL(Elrr(fl+1:nz-1,ii),r(fl+1:nz-1),[(r(fl+1)+r(fl))/2.])
      Eltout(ii)=INTERPOL(Eltt(fl+1:nz-1,ii),r(fl+1:nz-1),[(r(fl+1)+r(fl))/2.])
      
      Brin(ii)=INTERPOL(Bmrr(0:fl,ii),r(0:fl),[(r(fl+1)+r(fl))/2.])
      Btin(ii)=INTERPOL(bmtt(0:fl,ii),r(0:fl),[(r(fl+1)+r(fl))/2.])
      Brout(ii)=INTERPOL(bmrr(fl+1:nz-1,ii),r(fl+1:nz-1),[(r(fl+1)+r(fl))/2.])
      Btout(ii)=INTERPOL(bmtt(fl+1:nz-1,ii),r(fl+1:nz-1),[(r(fl+1)+r(fl))/2.])

      grrs(ii)=INTERPOL(grr(*,ii),r,[(r(fl+1)+r(fl))/2.])
      gtts(ii)=INTERPOL(gtt(*,ii),r,[(r(fl+1)+r(fl))/2.])
      omgs(ii)=INTERPOL(omgmet(*,ii),r,[(r(fl+1)+r(fl))/2.])
      alfns(ii)=INTERPOL(alfn(*,ii),r,[(r(fl+1)+r(fl))/2.])
      vphns(ii)=INTERPOL(vphn(0:fl,ii),r(0:fl),[(r(fl+1)+r(fl))/2.])
      psins(ii)=INTERPOL(psin(*,ii),r,[(r(fl+1)+r(fl))/2.])
    endfor

     Punit=1

    qsurf=rsup
    qsurf=(Elrout-Elrin)/grrs ;Carica superficiale
    JJsurf=0.*(Btout-Btin)/psins^2;*sin(th) !nulla per definizione
    
    if(punit eq 1) then begin
    p1=plot(th/acos(-1),qsurf*6.64674e18/1.e11,'-2r',title='Surface Charge Density',$
      xtitle='$\theta/\pi$',ytitle='$\rho_e [10^{11} statC/cm^2]$',Font_size=fsz,position=[0.15,0.1,0.9,0.5])
    endif  else begin
        p1=plot(th/acos(-1),qsurf,'-2b',title='Surface Charge Density',$
          xtitle='$\theta/\pi$',ytitle='$\rho_e$',Font_size=fsz)
    endelse

    if(punit eq 1) then dyne=5.55155e15 ; 10^23 dyne
    if(punit ne 1) then dyne=1.
    
    Lfr=qsurf*Elrout/grrs      ;Componente radiale della forza di Lorentz esterna
    Lft=qsurf*Eltout/gtts      ;Componente azimutale della forza di Lorentz esterna
    Lfrin=qsurf*Elrin/grrs     ;Componente radiale della forza di Lorentz interna
    Lftin=qsurf*Eltin/gtts     ;Componente azimutale della forza di Lorentz interna
        
    Lfb=qsurf*(Elrout*Brout+Eltout*Btout)/sqrt(Btout^2*gtts+Brout^2*grrs) ;Lorentz parallelo a B esterno
    Lfbin=qsurf*(Elrin*Brin+Eltin*Btin)/sqrt(Btin^2*gtts+Brin^2*grrs)     ;Lorentz parallelo a B interno
    oLfbr=Lfr-Lfb*Brout/sqrt(Btout^2*gtts+Brout^2*grrs)                       
    oLfbt=Lft-Lfb*Btout/sqrt(Btout^2*gtts+Brout^2*grrs)
    oLfbrin=Lfrin-Lfbin*Brin/sqrt(Btin^2*gtts+Brin^2*grrs)                       
    oLfbtin=Lftin-Lfbin*Btin/sqrt(Btin^2*gtts+Brin^2*grrs)
    oLfb=sqrt(oLfbt^2*gtts+oLfbr^2*grrs) ;Lorentz ortogonale a B esterno
    oLfbin=sqrt(oLfbtin^2*gtts+oLfbrin^2*grrs)

    pvec=vector(Exx,Eyy,rx,ry,RGB_TABLE=39, AUTO_COLOR=1,/auto_subsample,HEAD_SIZE=0.5,ASPECT_RATIO=1)
    ;exxres=exx
    ;eyyres=eyy
    ;exxres(where(rx2d^2+ry2d^2<10^2))=0.
    ;eyyres(where(rx2d^2+ry2d^2<10^2))=0.
    ;plot_field_new, exxres/sqrt(exxres^2+eyyres^2+1.e-20), eyyres/sqrt(exxres^2+eyyres^2+1.e-20),rx,ry,n=50,length=0.5,pos=pos1,title='',charsize=0.0001
    
    p1=plot(th/acos(-1),Lfr*dyne,'-2r',name='exterior',title='LorenzForce-radial',$
            xtitle='$\theta/\pi$',ytitle='$L^r [10^{23} dyne/cm]$',Font_size=fsz)
    p2=plot(th/acos(-1),Lfrin*dyne,'-2b',name='interior',/overplot)
    l=legend(target=[p1,p2])
        
    p1=plot(th/acos(-1),Lft*dyne,'-2r',name='exterior',title='LorenzForce - azimuthal',$
            xtitle='$\theta/\pi$',ytitle='$L^\theta [10^{23} dyne/cm]$',Font_size=fsz)
    p2=plot(th/acos(-1),Lftin*dyne,'-2b',name='interior',/overplot)
    l=legend(target=[p1,p2])

    p1=plot(th/acos(-1),Lfb*dyne,'-2r',name='exterior',title='$L parallel$',$
            xtitle='$\theta/\pi$',ytitle='$L// [10^{23} dyne/cm]$',Font_size=fsz)
    p2=plot(th/acos(-1),Lfbin*dyne,'-2b',name='interior',/overplot)
    l=legend(target=[p1,p2])

    p1=plot(th/acos(-1),oLfb*dyne,'-2r',name='exterior',title='$L perpendicular',$
            xtitle='$\theta/\pi$',ytitle='$L\perp [10^{23} dyne/cm]$',Font_size=fsz)
    p2=plot(th/acos(-1),oLfbin*dyne,'-2b',name='interior',/overplot)
    l=legend(target=[p1,p2])
    ;--------------------------------------------------------------------

    ;Stima della corrente aggiuntiva
    curtest=-qsurf*(omgs-omg)/alfns
    Ftestr=-(psins^6*rsup^2*sin(th))*curtest*btin/grrs
    Ftestt=(psins^6*rsup^2*sin(th))*curtest*brin/gtts
        
    Ftbin=(grrs*Ftestr*Brin+gtts*Ftestt*Btin)/sqrt(Btin^2*gtts+Brin^2*grrsin)
    ortr=Ftestr-FTbin*Brin/sqrt(Btin^2*gtts+Brin^2*grrs)                       
    ortt=Ftestt-FTbin*Btin/sqrt(Btin^2*gtts+Brin^2*grrs)
    ort=sqrt(ortr^2*grrs+ortt^2*gtts)
    
   ;---------------------------------------------------------------------
   ;Confronto
   ;---------------------------------------------------------------------

    openr,lun,'Hydroeq_mag2.dat',ERROR=err1
    if (err1 eq 0) then begin
      readf,lun,nx,nz
      bmrr2=dblarr(nz,nx)
      bmtt2=dblarr(nz,nx)
      bmpp2=dblarr(nz,nx)
      aphi2=dblarr(nz,nx)
      
      for ix=0,nx-1 do begin
         for iz=0,nz-1 do begin
            readf,lun,dat1,dat2,dat3,dat4
            bmpp2(iz,ix)=dat1      
            bmrr2(iz,ix)=dat2
            bmtt2(iz,ix)=dat3      
            aphi2(iz,ix)=dat4
         endfor
      endfor
   endif
   close,lun
   openr,lun,'Hydroeq_ele2.dat',ERROR=err2
   if (err2 eq 0) then begin
      readf,lun,nx,nz,kbpol
      elrr2=dblarr(nz,nx)
      eltt2=dblarr(nz,nx)
      atim2=dblarr(nz,nx)
      rhoe2=dblarr(nz,nx)
      Jphi2=dblarr(nz,nx)
      for ix=0,nx-1 do begin
         for iz=0,nz-1 do begin
            readf,lun,dat1,dat2,dat3,dat4,dat5
            elrr2(iz,ix)=dat1
            eltt2(iz,ix)=dat2
            atim2(iz,ix)=dat3
            rhoe2(iz,ix)=dat4
            Jphi2(iz,ix)=dat5
         endfor
      endfor
   endif
   close,lun

   if(err1 eq 0 and err2 eq 0) then begin
      e2pol2=Elrr2^2/grr+eltt2^2/gtt

      for ii=0,2*nxi-1 do begin
         for jj=0,2*nxi-1 do begin
            rad=sqrt(rx(ii)^2+ry(jj)^2)
            ang=acos(rx(jj)/rad)
            iang=fix(nx*ang/3.1415)
            irad=fix(nz*rad/max(r))
            if(irad gt nz-1)then irad=nz-1
            if(rx(ii) gt 0) then begin
               Exx2(ii,jj)=Elrr2(irad,iang)/sqrt(grr(irad,iang))*sin(ang)+Eltt2(irad,iang)/sqrt(gtt(irad,iang))*cos(ang)
               Eyy2(ii,jj)=Elrr2(irad,iang)/sqrt(grr(irad,iang))*cos(ang)-Eltt2(irad,iang)/sqrt(gtt(irad,iang))*sin(ang)
               arr(ii,jj)=sqrt(e2pol2(irad,iang))
            endif else begin
               Exx2(ii,jj)=-Elrr2(irad,iang)/sqrt(grr(irad,iang))*sin(ang)-Eltt2(irad,iang)/sqrt(gtt(irad,iang))*cos(ang)
               Eyy2(ii,jj)=Elrr2(irad,iang)/sqrt(grr(irad,iang))*cos(ang)-Eltt2(irad,iang)/sqrt(gtt(irad,iang))*sin(ang)
               arr(ii,jj)=-sqrt(e2pol2(irad,iang))
            endelse
               ;arr(ii,jj)=sqrt(e2pol(irad,iang))
            arrcirc(ii,jj)=pren(irad,iang)
            arratim(ii,jj)=atim(irad,iang)
         endfor
      endfor

      Elrin2=rsup
      Elrout2=rsup
      Eltin2=rsup
      Eltout2=rsup
      Brout2=rsup
      Btout2=rsup
      Brin2=rsup
      Btin2=rsup
      Lfr2=rsup
      Lft2=rsup
      Lfb2=rsup
      
      for ii=0,nx-1 do begin
         fl=rf(ii)
         Elrin2(ii)=INTERPOL(Elrr2(0:fl,ii),r(0:fl),[(r(fl+1)+r(fl))/2.])
         Eltin2(ii)=INTERPOL(Eltt2(0:fl,ii),r(0:fl),[(r(fl+1)+r(fl))/2.])
         Elrout2(ii)=INTERPOL(Elrr2(fl+1:nz-1,ii),r(fl+1:nz-1),[(r(fl+1)+r(fl))/2.])
         Eltout2(ii)=INTERPOL(Eltt2(fl+1:nz-1,ii),r(fl+1:nz-1),[(r(fl+1)+r(fl))/2.])
         
         Brin2(ii)=INTERPOL(Bmrr2(0:fl-1,ii),r(0:fl-1),[(r(fl+1)+r(fl))/2.])
         Btin2(ii)=INTERPOL(bmtt2(0:fl-1,ii),r(0:fl-1),[(r(fl+1)+r(fl))/2.])
         Brout2(ii)=INTERPOL(bmrr2(fl+1:nz-1,ii),r(fl+1:nz-1),[(r(fl+1)+r(fl))/2.])
         Btout2(ii)=INTERPOL(bmtt2(fl+1:nz-1,ii),r(fl+1:nz-1),[(r(fl+1)+r(fl))/2.])
      endfor
      
      qsurf2=rsup
      qsurf2=(Elrout2-Elrin2)/grrs ;Carica superficiale
      JJsurf2=-(Btout2-Btin2)/psins^2/sin(th) 
      
      qsurf3=qsurf+qsurf2
      Elrout3=Elrout+Elrout2
      Elrin3=Elrin+Elrin2
      Eltout3=Eltout+Eltout2
      Eltin3=Eltin+Eltin2
      
      brin3=brin+brin2
      brout3=brout+brout2
      btin3=btin+btin2
      btout3=btout+btout2
      
      Lfr2=qsurf3*Elrout3/grrs-(psins^6*rsup^2*sin(th))*JJsurf2*btout3/grrs 
      Lft2=qsurf3*Eltout3/gtts+(psins^6*rsup^2*sin(th))*JJsurf2*brout3/gtts 
      Lfrin2=qsurf3*Elrin3/grrs- (psins^6*rsup^2*sin(th))*JJsurf2*btin3/grrs 
      Lftin2=qsurf3*Eltin3/gtts+ (psins^6*rsup^2*sin(th))*JJsurf2*brin3/gtts 
      
      Lfb2=(Lfr2*Brout3*grrs+Lft2*Btout3*gtts)/sqrt(Btout3^2*gtts+Brout3^2*grrs)       ;Lorentz parallelo a B esterno
      Lfbin2=(Lfrin2*Brin3*grrs+Lftin2*Btin3*gtts)/sqrt(Btin3^2*gtts+Brin3^2*grrs)     ;Lorentz parallelo a B interno
      oLfbr2=Lfr2-Lfb2*Brout3/sqrt(Btout3^2*gtts+Brout3^2*grrs)                       
      oLfbt2=Lft2-Lfb2*Btout3/sqrt(Btout3^2*gtts+Brout3^2*grrs)
      oLfbrin2=Lfrin2-Lfbin2*Brin3/sqrt(Btin3^2*gtts+Brin3^2*grrs)                       
      oLfbtin2=Lftin2-Lfbin2*Btin3/sqrt(Btin3^2*gtts+Brin3^2*grrs)
      oLfb2=sqrt(oLfbt2^2*gtts+oLfbr2^2*grrs)   ;Lorentz ortogonale a B esterno
      oLfbin2=sqrt(oLfbtin2^2*gtts+oLfbrin2^2*grrs)
      
      ;----------------sfilza di plot-------------------------------------
      pvec=vector(Exx+Exx2,Eyy+Eyy2,rx,ry,RGB_TABLE=39, AUTO_COLOR=1,/auto_subsample,HEAD_SIZE=0.4,$
        LENGTH_SCALE=0.5,ASPECT_RATIO=1)
      
      p1=plot(th/acos(-1),Lfr2*dyne,'-2r',name='exterior',title='LorenzForce-radial (with surface current)',$
              xtitle='$\theta/\pi$',ytitle='$L^r [10^{23} dyne/cm]$',Font_size=fsz)
      p2=plot(th/acos(-1),Lfrin2*dyne,'-2b',name='interior',/overplot)
      l=legend(target=[p1,p2])
      
      p1=plot(th/acos(-1),Lft2*dyne,'-2r',name='exterior',title='LorenzForce - azimuthal (with surface current)',$
              xtitle='$\theta/\pi$',ytitle='$L^\theta [10^{23} dyne/cm]$',Font_size=fsz)
      p2=plot(th/acos(-1),Lftin2*dyne,'-2b',name='interior',/overplot)
      l=legend(target=[p1,p2])
      
      
      p1=plot(th/acos(-1),Lfb2*dyne,'-2r',name='exterior',title='$L\cdot B / B^2$ (with surface current)',$
              xtitle='$\theta/\pi$',ytitle='$L// [10^{23} dyne/cm]$',Font_size=fsz)
      p2=plot(th/acos(-1),Lfbin2*dyne,'-2b',name='interior',/overplot)
      l=legend(target=[p1,p2])
      
      p1=plot(th/acos(-1),Lfb2*dyne,'-2r',name='exterior',title='$L\cdot B / B^2$ (with surface current)',$
        xtitle='$\theta/\pi$',ytitle='$L// [10^{23} dyne/cm]$',Font_size=fsz)
      
      p1=plot(th/acos(-1),oLfb2*dyne,'-2r',name='exterior',title='$L-L\cdot B / B^2$ (with surface current)',$
              xtitle='$\theta/\pi$',ytitle='$L\perp [10^{23} dyne/cm]$',Font_size=fsz)
      p2=plot(th/acos(-1),oLfbin2*dyne,'-2b',name='interior',/overplot)
      l=legend(target=[p1,p2])
      
      
      p1=plot(th/acos(-1),Lfb2*dyne,'-2b',name='exterior',title='$L\cdot B / B^2$ (with surface current)',$
        xtitle='$\theta/\pi$',ytitle='$L// [10^{23} dyne/cm]$',Font_size=fsz,position=[0.15,0.1,0.9,0.5])
      p1=plot(th/acos(-1),oLfb2*dyne,'-2g',name='exterior',title='$L-L\cdot B / B^2$ (with surface current)',$
        xtitle='$\theta/\pi$',ytitle='$L\perp [10^{23} dyne/cm]$',Font_size=fsz,position=[0.15,0.1,0.9,0.5])
     ;--------------------------------------------------------------------
      endif
  endif

if (output eq 1) then begin
    device,/close
    set_plot,'x'
 endif  
      
         
end
